@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -19,7 +20,7 @@
  *              This worker is deprecated                                  *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Based on:    jserv_ajpv12.c from Jserv                                  *
- * Version:     $Revision: 422996 $                                          *
+ * Version:     $Revision: 603324 $                                          *
  ***************************************************************************/
 
 #include "jk_ajp12_worker.h"
@@ -27,7 +28,7 @@
 #include "jk_connect.h"
 #include "jk_util.h"
 #include "jk_sockbuf.h"
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
 #include "util_ebcdic.h"
 #include <string.h>
 #endif
@@ -60,11 +61,11 @@ typedef struct ajp12_endpoint ajp12_endpoint_t;
 
 static int ajpv12_mark(ajp12_endpoint_t * p, unsigned char type);
 
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
 static int ajpv12_sendasciistring(ajp12_endpoint_t * p, char *buffer);
 #endif
 
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
 static int ajpv12_sendstring(ajp12_endpoint_t * p, char *buffer);
 #else
 static int ajpv12_sendstring(ajp12_endpoint_t * p, const char *buffer);
@@ -83,49 +84,73 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
 static int ajpv12_handle_request(ajp12_endpoint_t * p,
                                  jk_ws_service_t *s, jk_logger_t *l);
 
+/*
+ * Return values of service() method for ajp12 worker:
+ * return value  is_error              reason
+ * JK_FALSE      JK_HTTP_SERVER_ERROR  Invalid parameters (null values)
+ *                                     Error during connect to the backend
+ *                                     ajpv12_handle_request() returns false:
+ *           Any error during reading a request body from the client or
+ *           sending the request to the backend
+ * JK_FALSE      JK_HTTP_OK            ajpv12_handle_response() returns false:
+ *           Any error during reading parts of response from backend or
+ *           sending to client
+ * JK_TRUE       JK_HTTP_OK            All other cases
+ */
 static int JK_METHOD service(jk_endpoint_t *e,
                              jk_ws_service_t *s,
                              jk_logger_t *l, int *is_error)
 {
-    jk_log(l, JK_LOG_DEBUG, "Into jk_endpoint_t::service");
+    ajp12_endpoint_t *p;
+    unsigned int attempt;
+    int rc = -1;
+    /*
+     * AJP12 protocol is not recoverable.
+     */
 
-    if (e && e->endpoint_private && s && is_error) {
-        ajp12_endpoint_t *p = e->endpoint_private;
-        unsigned int attempt;
-        /*
-         * AJP12 protocol is not recoverable.
-         */
-        *is_error = JK_HTTP_SERVER_ERROR;
+    JK_TRACE_ENTER(l);
 
-        for (attempt = 0; attempt < p->worker->connect_retry_attempts;
-             attempt++) {
-            p->sd =
-                jk_open_socket(&p->worker->worker_inet_addr,
-                               JK_FALSE, -1, 0, l);
+    if (!e || !e->endpoint_private || !s || !is_error) {
+        JK_LOG_NULL_PARAMS(l);
+        if (is_error)
+            *is_error = JK_HTTP_SERVER_ERROR;
+        JK_TRACE_EXIT(l);
+        return JK_FALSE;
+    }
 
-            jk_log(l, JK_LOG_DEBUG, "In jk_endpoint_t::service, sd = %d",
-                   p->sd);
-            if (IS_VALID_SOCKET(p->sd)) {
-                break;
-            }
-        }
-        if (IS_VALID_SOCKET(p->sd)) {
+    p = e->endpoint_private;
 
-            jk_sb_open(&p->sb, p->sd);
-            if (ajpv12_handle_request(p, s, l)) {
-                jk_log(l, JK_LOG_DEBUG,
-                       "In jk_endpoint_t::service, sent request");
-                return ajpv12_handle_response(p, s, l);
-            }
-        }
-        jk_log(l, JK_LOG_ERROR, "In jk_endpoint_t::service, Error sd = %d",
+    /* Set returned error to OK */
+    *is_error = JK_HTTP_OK;
+
+    for (attempt = 0; attempt < p->worker->connect_retry_attempts;
+         attempt++) {
+        p->sd =
+            jk_open_socket(&p->worker->worker_inet_addr,
+                           JK_FALSE, 0, 0, l);
+
+        jk_log(l, JK_LOG_DEBUG, "In jk_endpoint_t::service, sd = %d",
                p->sd);
+        if (IS_VALID_SOCKET(p->sd)) {
+            break;
+        }
     }
-    else {
-        jk_log(l, JK_LOG_ERROR,
-               "In jk_endpoint_t::service, NULL parameters");
-    }
+    if (IS_VALID_SOCKET(p->sd)) {
 
+        jk_sb_open(&p->sb, p->sd);
+        if (ajpv12_handle_request(p, s, l)) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "In jk_endpoint_t::service, sent request");
+            rc = ajpv12_handle_response(p, s, l);
+            JK_TRACE_EXIT(l);
+            return rc;
+        }
+    }
+    jk_log(l, JK_LOG_ERROR, "In jk_endpoint_t::service, Error sd = %d",
+           p->sd);
+    *is_error = JK_HTTP_SERVER_ERROR;
+
+    JK_TRACE_EXIT(l);
     return JK_FALSE;
 }
 
@@ -135,7 +160,7 @@ static int JK_METHOD done(jk_endpoint_t **e, jk_logger_t *l)
     if (e && *e && (*e)->endpoint_private) {
         ajp12_endpoint_t *p = (*e)->endpoint_private;
         if (IS_VALID_SOCKET(p->sd)) {
-            jk_close_socket(p->sd);
+            jk_shutdown_socket(p->sd, l);
         }
         free(p);
         *e = NULL;
@@ -167,7 +192,7 @@ static int JK_METHOD validate(jk_worker_t *pThis,
                p->name, host, port);
 
         if (port > 1024 && host) {
-            if (jk_resolve(host, port, &p->worker_inet_addr)) {
+            if (jk_resolve(host, port, &p->worker_inet_addr, l)) {
                 return JK_TRUE;
             }
             jk_log(l, JK_LOG_ERROR,
@@ -295,7 +320,7 @@ static int ajpv12_sendnbytes(ajp12_endpoint_t * p,
     }
 }
 
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
 static int ajpv12_sendasciistring(ajp12_endpoint_t * p, const char *buffer)
 {
     int bufferlen;
@@ -313,8 +338,8 @@ static int ajpv12_sendstring(ajp12_endpoint_t * p, const char *buffer)
 {
     int bufferlen;
 
-    if (buffer && (bufferlen = strlen(buffer))) {
-#if defined(AS400) || defined(_OSD_POSIX)
+    if (buffer && (bufferlen = (int)strlen(buffer))) {
+#if (defined(AS400) && !defined(AS400_UTF8)) || defined(_OSD_POSIX)
         char buf[2048];
         if (bufferlen < 2048) {
             memcpy(buf, buffer, bufferlen);
@@ -371,7 +396,7 @@ static int ajpv12_handle_request(ajp12_endpoint_t * p,
            ajpv12_sendstring(p, s->server_name) && ajpv12_sendstring(p, 0) &&   /* doc root */
            ajpv12_sendstring(p, 0) &&   /* path info */
            ajpv12_sendstring(p, 0) &&   /* path translated */
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
            ajpv12_sendasciistring(p, s->query_string) &&
 #else
            ajpv12_sendstring(p, s->query_string) &&
@@ -381,20 +406,20 @@ static int ajpv12_handle_request(ajp12_endpoint_t * p,
            ajpv12_sendstring(p, s->remote_user) &&
            ajpv12_sendstring(p, s->auth_type) &&
            ajpv12_sendint(p, s->server_port) &&
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
            ajpv12_sendasciistring(p, s->method) &&
 #else
            ajpv12_sendstring(p, s->method) &&
 #endif
            ajpv12_sendstring(p, s->req_uri) && ajpv12_sendstring(p, 0) &&       /* */
            ajpv12_sendstring(p, 0) &&   /* SCRIPT_NAME */
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
            ajpv12_sendasciistring(p, s->server_name) &&
 #else
            ajpv12_sendstring(p, s->server_name) &&
 #endif
            ajpv12_sendint(p, s->server_port) && ajpv12_sendstring(p, s->protocol) && ajpv12_sendstring(p, 0) && /* SERVER_SIGNATURE */
-           ajpv12_sendstring(p, s->server_software) && ajpv12_sendstring(p, s->jvm_route) &&    /* JSERV_ROUTE */
+           ajpv12_sendstring(p, s->server_software) && ajpv12_sendstring(p, s->route) &&    /* JSERV_ROUTE */
            ajpv12_sendstring(p, "") &&  /* JSERV ajpv12 compatibility */
            ajpv12_sendstring(p, ""));   /* JSERV ajpv12 compatibility */
 
@@ -451,17 +476,20 @@ static int ajpv12_handle_request(ajp12_endpoint_t * p,
 
     if (s->content_length) {
         char buf[READ_BUF_SIZE];
-        unsigned so_far = 0;
+        jk_uint64_t so_far = 0;
 
         jk_log(l, JK_LOG_DEBUG,
                "ajpv12_handle_request, sending the request body");
 
         while (so_far < s->content_length) {
             unsigned this_time = 0;
-            unsigned to_read = s->content_length - so_far;
-            if (to_read > READ_BUF_SIZE) {
+            unsigned to_read;
+            if (s->content_length > so_far + READ_BUF_SIZE) {
                 to_read = READ_BUF_SIZE;
             }
+            else {
+                to_read = (unsigned int)(s->content_length - so_far);
+            }
 
             if (!s->read(s, buf, to_read, &this_time)) {
                 jk_log(l, JK_LOG_ERROR,
@@ -482,7 +510,7 @@ static int ajpv12_handle_request(ajp12_endpoint_t * p,
             }
             else if (this_time == 0) {
                 jk_log(l, JK_LOG_ERROR,
-                       "In ajpv12_handle_request, Error: short read. content length is %d, read %d",
+                       "In ajpv12_handle_request, Error: short read. content length is %" JK_UINT64_T_FMT ", read %" JK_UINT64_T_FMT,
                        s->content_length, so_far);
                 return JK_FALSE;
             }
@@ -512,7 +540,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
         char *line = NULL;
         char *name = NULL;
         char *value = NULL;
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
         char *lasts;
 #endif
 
@@ -521,7 +549,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
                    "ajpv12_handle_response, error reading header line");
             return JK_FALSE;
         }
-#if defined(AS400) || defined(_OSD_POSIX)
+#if (defined(AS400) && !defined(AS400_UTF8)) || defined(_OSD_POSIX)
         jk_xlate_from_ascii(line, strlen(line));
 #endif
 
@@ -533,7 +561,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
         }
 
         name = line;
-        while (isspace(*name) && *name) {
+        while (isspace((int)(*name)) && *name) {
             name++;             /* Skip leading white chars */
         }
         if (!*name) {           /* Empty header name */
@@ -548,7 +576,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
         }
         *value = '\0';
         value++;
-        while (isspace(*value) && *value) {
+        while (isspace((int)(*value)) && *value) {
             value++;            /* Skip leading white chars */
         }
         if (!*value) {          /* Empty header value */
@@ -560,7 +588,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
         jk_log(l, JK_LOG_DEBUG, "ajpv12_handle_response, read %s=%s", name,
                value);
         if (0 == strcmp("Status", name)) {
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
             char *numeric = strtok_r(value, " \t", &lasts);
 #else
             char *numeric = strtok(value, " \t");
@@ -572,7 +600,7 @@ static int ajpv12_handle_response(ajp12_endpoint_t * p,
                        "ajpv12_handle_response, invalid status code");
                 return JK_FALSE;
             }
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
             reason = jk_pool_strdup(s->pool, strtok_r(NULL, " \t", &lasts));
 #else
             reason = jk_pool_strdup(s->pool, strtok(NULL, " \t"));
