@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -22,7 +23,7 @@
  * @package jk_connect
  * @author      Gal Shachor <shachor@il.ibm.com>
  * @author      Mladen Turk <mturk@apache.org>
- * @version     $Revision: 423000 $
+ * @version     $Revision: 603677 $
  */
 
 
@@ -37,6 +38,11 @@
 static apr_pool_t *jk_apr_pool = NULL;
 #endif
 
+#ifdef HAVE_SYS_FILIO_H
+/* FIONREAD on Solaris et al. */
+#include <sys/filio.h>
+#endif
+
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
 #define JK_IS_SOCKET_ERROR(x) ((x) == SOCKET_ERROR)
 #define JK_GET_SOCKET_ERRNO() errno = WSAGetLastError() - WSABASEERR
@@ -52,7 +58,13 @@ typedef char* SET_TYPE;
 typedef const char* SET_TYPE;
 #endif
 
-static int soblock(int sd)
+/** Set socket to blocking
+ * @param sd  socket to manipulate
+ * @return    errno: fcntl returns -1 (!WIN32)
+ *            pseudo errno: ioctlsocket returns SOCKET_ERROR (WIN32)
+ *            0: success
+ */
+static int soblock(jk_sock_t sd)
 {
 /* BeOS uses setsockopt at present for non blocking... */
 #ifndef WIN32
@@ -73,14 +85,20 @@ static int soblock(int sd)
     }
 #else
     u_long on = 0;
-    if (ioctlsocket(sd, FIONBIO, &on) == SOCKET_ERROR) {
-        errno = WSAGetLastError() - WSABASEERR;
+    if (JK_IS_SOCKET_ERROR(ioctlsocket(sd, FIONBIO, &on))) {
+        JK_GET_SOCKET_ERRNO();
         return errno;
     }
 #endif /* WIN32 */
     return 0;
 }
 
+/** Set socket to non-blocking
+ * @param sd  socket to manipulate
+ * @return    errno: fcntl returns -1 (!WIN32)
+ *            pseudo errno: ioctlsocket returns SOCKET_ERROR (WIN32)
+ *            0: success
+ */
 static int sononblock(jk_sock_t sd)
 {
 #ifndef WIN32
@@ -101,8 +119,8 @@ static int sononblock(jk_sock_t sd)
     }
 #else
     u_long on = 1;
-    if (ioctlsocket(sd, FIONBIO, &on) == SOCKET_ERROR) {
-        errno = WSAGetLastError() - WSABASEERR;
+    if (JK_IS_SOCKET_ERROR(ioctlsocket(sd, FIONBIO, &on))) {
+        JK_GET_SOCKET_ERRNO();
         return errno;
     }
 #endif /* WIN32 */
@@ -111,66 +129,102 @@ static int sononblock(jk_sock_t sd)
 
 #if defined (WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
 /* WIN32 implementation */
-static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
+/** Non-blocking socket connect
+ * @param sd       socket to connect
+ * @param addr     address to connect to
+ * @param timeout  connect timeout in seconds
+ *                 (<=0: no timeout=blocking)
+ * @param l        logger
+ * @return         -1: some kind of error occured
+ *                 SOCKET_ERROR: no timeout given and error
+ *                               during blocking connect
+ *                 0: success
+ */
+static int nb_connect(jk_sock_t sd, struct sockaddr *addr, int timeout, jk_logger_t *l)
 {
     int rc;
-    if (timeout < 1)
-        return connect(sock, addr, sizeof(struct sockaddr_in));
 
-    if ((rc = sononblock(sock)))
+    JK_TRACE_ENTER(l);
+
+    if (timeout <= 0) {
+        rc = connect(sd, addr, sizeof(struct sockaddr_in));
+        JK_TRACE_EXIT(l);
+        return rc;
+    }
+
+    if ((rc = sononblock(sd))) {
+        JK_TRACE_EXIT(l);
         return -1;
-    if (connect(sock, addr, sizeof(struct sockaddr_in)) == SOCKET_ERROR) {
+    }
+    if (JK_IS_SOCKET_ERROR(connect(sd, addr, sizeof(struct sockaddr_in)))) {
         struct timeval tv;
         fd_set wfdset, efdset;
 
         if ((rc = WSAGetLastError()) != WSAEWOULDBLOCK) {
-            soblock(sock);
+            soblock(sd);
             WSASetLastError(rc);
+            JK_TRACE_EXIT(l);
             return -1;
         }
         /* wait for the connect to complete or timeout */
         FD_ZERO(&wfdset);
-        FD_SET(sock, &wfdset);
+        FD_SET(sd, &wfdset);
         FD_ZERO(&efdset);
-        FD_SET(sock, &efdset);
+        FD_SET(sd, &efdset);
 
         tv.tv_sec  = timeout;
         tv.tv_usec = 0;
-        rc = select(sock + 1, NULL, &wfdset, &efdset, &tv);
-        if (rc == SOCKET_ERROR || rc == 0) {
+        rc = select((int)sd + 1, NULL, &wfdset, &efdset, &tv);
+        if (JK_IS_SOCKET_ERROR(rc) || rc == 0) {
             rc = WSAGetLastError();
-            soblock(sock);
+            soblock(sd);
             WSASetLastError(rc);
+            JK_TRACE_EXIT(l);
             return -1;
         }
         /* Evaluate the efdset */
-        if (FD_ISSET(sock, &efdset)) {
+        if (FD_ISSET(sd, &efdset)) {
             /* The connect failed. */
             int rclen = (int)sizeof(rc);
-            if (getsockopt(sock, SOL_SOCKET, SO_ERROR, (char*) &rc, &rclen))
+            if (getsockopt(sd, SOL_SOCKET, SO_ERROR, (char*) &rc, &rclen))
                 rc = 0;
-            soblock(sock);
+            soblock(sd);
             if (rc)
                 WSASetLastError(rc);
+            JK_TRACE_EXIT(l);
             return -1;
         }
     }
-    soblock(sock);
+    soblock(sd);
+    JK_TRACE_EXIT(l);
     return 0;
 }
 
 #elif !defined(NETWARE)
 /* POSIX implementation */
-static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
+/** Non-blocking socket connect
+ * @param sd       socket to connect
+ * @param addr     address to connect to
+ * @param timeout  connect timeout in seconds
+ *                 (<=0: no timeout=blocking)
+ * @param l        logger
+ * @return         -1: some kind of error occured
+ *                 0: success
+ */
+static int nb_connect(jk_sock_t sd, struct sockaddr *addr, int timeout, jk_logger_t *l)
 {
     int rc = 0;
 
+    JK_TRACE_ENTER(l);
+
     if (timeout > 0) {
-        if (sononblock(sock))
+        if (sononblock(sd)) {
+            JK_TRACE_EXIT(l);
             return -1;
+        }
     }
     do {
-        rc = connect(sock, addr, sizeof(struct sockaddr_in));
+        rc = connect(sd, addr, sizeof(struct sockaddr_in));
     } while (rc == -1 && errno == EINTR);
 
     if ((rc == -1) && (errno == EINPROGRESS || errno == EALREADY)
@@ -180,21 +234,22 @@ static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
         socklen_t rclen = (socklen_t)sizeof(rc);
 
         FD_ZERO(&wfdset);
-        FD_SET(sock, &wfdset);
+        FD_SET(sd, &wfdset);
         tv.tv_sec  = timeout;
         tv.tv_usec = 0;
-        rc = select(sock + 1, NULL, &wfdset, NULL, &tv);
+        rc = select(sd + 1, NULL, &wfdset, NULL, &tv);
         if (rc <= 0) {
             /* Save errno */
             int err = errno;
-            soblock(sock);
+            soblock(sd);
             errno = err;
+            JK_TRACE_EXIT(l);
             return -1;
         }
         rc = 0;
 #ifdef SO_ERROR
-        if (!FD_ISSET(sock, &wfdset) ||
-            (getsockopt(sock, SOL_SOCKET, SO_ERROR,
+        if (!FD_ISSET(sd, &wfdset) ||
+            (getsockopt(sd, SOL_SOCKET, SO_ERROR,
                         (char *)&rc, &rclen) < 0) || rc) {
             if (rc)
                 errno = rc;
@@ -205,24 +260,68 @@ static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
     /* Not sure we can be already connected */
     if (rc == -1 && errno == EISCONN)
         rc = 0;
-    soblock(sock);
+    soblock(sd);
+    JK_TRACE_EXIT(l);
     return rc;
 }
 #else
 /* NETWARE implementation - blocking for now */
-static int nb_connect(jk_sock_t sock, struct sockaddr *addr, int timeout)
+/** Non-blocking socket connect
+ * @param sd       socket to connect
+ * @param addr     address to connect to
+ * @param timeout  connect timeout in seconds (ignored!)
+ * @param l        logger
+ * @return         -1: some kind of error occured
+ *                 0: success
+ */
+static int nb_connect(jk_sock_t sd, struct sockaddr *addr, int timeout, jk_logger_t *l)
 {
-    return connect(sock, addr, sizeof(struct sockaddr_in));
+    int rc;
+
+    JK_TRACE_ENTER(l);
+
+    rc = connect(sd, addr, sizeof(struct sockaddr_in));
+    JK_TRACE_EXIT(l);
+    return rc;
 }
 #endif
 
-/** resolve the host IP */
 
-int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
+#ifdef AS400_UTF8
+
+/*
+ *  i5/OS V5R4 need EBCDIC for its runtime calls but APR/APACHE works in UTF
+ */
+in_addr_t jk_inet_addr(const char * addrstr)
+{
+    in_addr_t addr;
+    char *ptr;
+
+    ptr = (char *)malloc(strlen(addrstr) + 1);
+    jk_ascii2ebcdic((char *)addrstr, ptr);
+    addr = inet_addr(ptr);
+    free(ptr);
+
+    return(addr);
+}
+
+#endif
+
+/** Resolve the host IP
+ * @param host     host or ip address
+ * @param port     port
+ * @param rc       return value pointer
+ * @param l        logger
+ * @return         JK_FALSE: some kind of error occured
+ *                 JK_TRUE: success
+ */
+int jk_resolve(const char *host, int port, struct sockaddr_in *rc, jk_logger_t *l)
 {
     int x;
     struct in_addr laddr;
 
+    JK_TRACE_ENTER(l);
+
     memset(rc, 0, sizeof(struct sockaddr_in));
 
     rc->sin_port = htons((short)port);
@@ -230,12 +329,12 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
 
     /* Check if we only have digits in the string */
     for (x = 0; host[x] != '\0'; x++) {
-        if (!isdigit(host[x]) && host[x] != '.') {
+        if (!isdigit((int)(host[x])) && host[x] != '.') {
             break;
         }
     }
 
-    /* If we found also characters we shoud make name to IP resolution */
+    /* If we found also characters we should make name to IP resolution */
     if (host[x] != '\0') {
 
 #ifdef HAVE_APR
@@ -243,13 +342,17 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
         char *remote_ipaddr;
 
         if (!jk_apr_pool) {
-            if (apr_pool_create(&jk_apr_pool, NULL) != APR_SUCCESS)
+            if (apr_pool_create(&jk_apr_pool, NULL) != APR_SUCCESS) {
+                JK_TRACE_EXIT(l);
                 return JK_FALSE;
+            }
         }
         if (apr_sockaddr_info_get
             (&remote_sa, host, APR_UNSPEC, (apr_port_t) port, 0, jk_apr_pool)
-            != APR_SUCCESS)
+            != APR_SUCCESS) {
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
+        }
 
         /* Since we are only handling AF_INET (IPV4) address (in_addr_t) */
         /* make sure we find one of those.                               */
@@ -260,11 +363,14 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
         /* if temp_sa is set, we have a valid address otherwise, just return */
         if (NULL != temp_sa)
             remote_sa = temp_sa;
-        else
+        else {
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
+        }
 
         apr_sockaddr_ip_get(&remote_ipaddr, remote_sa);
-        laddr.s_addr = inet_addr(remote_ipaddr);
+
+        laddr.s_addr = jk_inet_addr(remote_ipaddr);
 
 #else /* HAVE_APR */
 
@@ -276,6 +382,7 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
         struct hostent *hoste = gethostbyname(host);
 #endif
         if (!hoste) {
+            JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
 
@@ -285,20 +392,31 @@ int jk_resolve(const char *host, int port, struct sockaddr_in *rc)
     }
     else {
         /* If we found only digits we use inet_addr() */
-        laddr.s_addr = inet_addr(host);
+        laddr.s_addr = jk_inet_addr(host);
     }
     memcpy(&(rc->sin_addr), &laddr, sizeof(laddr));
 
+    JK_TRACE_EXIT(l);
     return JK_TRUE;
 }
 
-/** connect to Tomcat */
-
+/** Connect to Tomcat
+ * @param addr      address to connect to
+ * @param keepalive should we set SO_KEEPALIVE (if !=0)
+ * @param timeout   connect timeout in seconds
+ *                  (<=0: no timeout=blocking)
+ * @param sock_buf  size of send and recv buffer
+ *                  (<=0: use default)
+ * @param l         logger
+ * @return          JK_INVALID_SOCKET: some kind of error occured
+ *                  created socket: success
+ * @remark          Cares about errno
+ */
 jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
                          int timeout, int sock_buf, jk_logger_t *l)
 {
     char buf[32];
-    jk_sock_t sock;
+    jk_sock_t sd;
     int set = 1;
     int ret = 0;
 #ifdef SO_LINGER
@@ -307,21 +425,22 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
 
     JK_TRACE_ENTER(l);
 
-    sock = socket(AF_INET, SOCK_STREAM, 0);
-    if (!IS_VALID_SOCKET(sock)) {
+    errno = 0;
+    sd = socket(AF_INET, SOCK_STREAM, 0);
+    if (!IS_VALID_SOCKET(sd)) {
         JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
-               "socket() failed with errno=%d", errno);
+               "socket() failed (errno=%d)", errno);
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
-;
     }
     /* Disable Nagle algorithm */
-    if (setsockopt(sock, IPPROTO_TCP, TCP_NODELAY, (SET_TYPE)&set,
+    if (setsockopt(sd, IPPROTO_TCP, TCP_NODELAY, (SET_TYPE)&set,
                    sizeof(set))) {
+        JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
-                "failed setting TCP_NODELAY with errno=%d", errno);
-        jk_close_socket(sock);
+                "failed setting TCP_NODELAY (errno=%d)", errno);
+        jk_close_socket(sd, l);
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
     }
@@ -330,11 +449,12 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
                "socket TCP_NODELAY set to On");
     if (keepalive) {
         set = 1;
-        if (setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, (SET_TYPE)&set,
+        if (setsockopt(sd, SOL_SOCKET, SO_KEEPALIVE, (SET_TYPE)&set,
                        sizeof(set))) {
+            JK_GET_SOCKET_ERRNO();
             jk_log(l, JK_LOG_ERROR,
-                   "failed setting SO_KEEPALIVE with errno=%d", errno);
-            jk_close_socket(sock);
+                   "failed setting SO_KEEPALIVE (errno=%d)", errno);
+            jk_close_socket(sd, l);
             JK_TRACE_EXIT(l);
             return JK_INVALID_SOCKET;
         }
@@ -346,52 +466,53 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
     if (sock_buf > 0) {
         set = sock_buf;
         /* Set socket send buffer size */
-        if (setsockopt(sock, SOL_SOCKET, SO_SNDBUF, (SET_TYPE)&set,
+        if (setsockopt(sd, SOL_SOCKET, SO_SNDBUF, (SET_TYPE)&set,
                         sizeof(set))) {
             JK_GET_SOCKET_ERRNO();
             jk_log(l, JK_LOG_ERROR,
-                    "failed setting SO_SNDBUF with errno=%d", errno);
-            jk_close_socket(sock);
+                    "failed setting SO_SNDBUF (errno=%d)", errno);
+            jk_close_socket(sd, l);
             JK_TRACE_EXIT(l);
             return JK_INVALID_SOCKET;
         }
         set = sock_buf;
         /* Set socket receive buffer size */
-        if (setsockopt(sock, SOL_SOCKET, SO_RCVBUF, (SET_TYPE)&set,
+        if (setsockopt(sd, SOL_SOCKET, SO_RCVBUF, (SET_TYPE)&set,
                                 sizeof(set))) {
             JK_GET_SOCKET_ERRNO();
             jk_log(l, JK_LOG_ERROR,
-                    "failed setting SO_RCVBUF with errno=%d", errno);
-            jk_close_socket(sock);
+                    "failed setting SO_RCVBUF (errno=%d)", errno);
+            jk_close_socket(sd, l);
             JK_TRACE_EXIT(l);
             return JK_INVALID_SOCKET;
         }
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
-                   "socket SO_SNDBUF and  SO_RCVBUF set to %d",
+                   "socket SO_SNDBUF and SO_RCVBUF set to %d",
                    sock_buf);
     }
 
     if (timeout > 0) {
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
         int tmout = timeout * 1000;
-        setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO,
+        setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO,
                    (const char *) &tmout, sizeof(int));
-        setsockopt(sock, SOL_SOCKET, SO_SNDTIMEO,
+        setsockopt(sd, SOL_SOCKET, SO_SNDTIMEO,
                    (const char *) &tmout, sizeof(int));
+        JK_GET_SOCKET_ERRNO();
 #elif defined(SO_RCVTIMEO) && defined(USE_SO_RCVTIMEO) && defined(SO_SNDTIMEO) && defined(USE_SO_SNDTIMEO)
         struct timeval tv;
         tv.tv_sec  = timeout;
         tv.tv_usec = 0;
-        setsockopt(sock, SOL_SOCKET, SO_RCVTIMEO,
+        setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO,
                    (const void *) &tv, sizeof(tv));
-        setsockopt(sock, SOL_SOCKET, SO_SNDTIMEO,
+        setsockopt(sd, SOL_SOCKET, SO_SNDTIMEO,
                    (const void *) &tv, sizeof(tv));
 #endif
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG,
                    "timeout %d set for socket=%d",
-                   timeout, sock);
+                   timeout, sd);
     }
 #ifdef SO_NOSIGPIPE
     /* The preferred method on Mac OS X (10.2 and later) to prevent SIGPIPEs when
@@ -399,12 +520,12 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
      * systems?
     */
     set = 1;
-    if (setsockopt(sock, SOL_SOCKET, SO_NOSIGPIPE, (const char *)&set,
+    if (setsockopt(sd, SOL_SOCKET, SO_NOSIGPIPE, (const char *)&set,
                    sizeof(int))) {
         JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
-                "failed setting SO_NOSIGPIPE with errno=%d", errno);
-        jk_close_socket(sock);
+                "failed setting SO_NOSIGPIPE (errno=%d)", errno);
+        jk_close_socket(sd, l);
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
     }
@@ -412,12 +533,12 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
 #ifdef SO_LINGER
     /* Make hard closesocket by disabling lingering */
     li.l_linger = li.l_onoff = 0;
-    if (setsockopt(sock, SOL_SOCKET, SO_LINGER, (SET_TYPE)&li,
+    if (setsockopt(sd, SOL_SOCKET, SO_LINGER, (SET_TYPE)&li,
                    sizeof(li))) {
         JK_GET_SOCKET_ERRNO();
         jk_log(l, JK_LOG_ERROR,
-                "failed setting SO_LINGER with errno=%d", errno);
-        jk_close_socket(sock);
+                "failed setting SO_LINGER (errno=%d)", errno);
+        jk_close_socket(sd, l);
         JK_TRACE_EXIT(l);
         return JK_INVALID_SOCKET;
     }
@@ -425,7 +546,7 @@ jk_sock_t jk_open_socket(struct sockaddr_in *addr, int keepalive,
     /* Tries to connect to Tomcat (continues trying while error is EINTR) */
     if (JK_IS_DEBUG_LEVEL(l))
         jk_log(l, JK_LOG_DEBUG,
-                "trying to connect socket %d to %s", sock,
+                "trying to connect socket %d to %s", sd,
                 jk_dump_hinfo(addr, buf));
 
 /* Need more infos for BSD 4.4 and Unix 98 defines, for now only
@@ -433,49 +554,65 @@ iSeries when Unix98 is required at compil time */
 #if (_XOPEN_SOURCE >= 520) && defined(AS400)
     ((struct sockaddr *)addr)->sa_len = sizeof(struct sockaddr_in);
 #endif
-    ret = nb_connect(sock, (struct sockaddr *)addr, timeout);
+    ret = nb_connect(sd, (struct sockaddr *)addr, timeout, l);
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-    if (ret == SOCKET_ERROR) {
-        errno = WSAGetLastError() - WSABASEERR;
+    if (JK_IS_SOCKET_ERROR(ret)) {
+        JK_GET_SOCKET_ERRNO();
     }
 #endif /* WIN32 */
 
     /* Check if we are connected */
     if (ret) {
         jk_log(l, JK_LOG_INFO,
-               "connect to %s failed with errno=%d",
+               "connect to %s failed (errno=%d)",
                jk_dump_hinfo(addr, buf), errno);
-        jk_close_socket(sock);
-        sock = JK_INVALID_SOCKET;
+        jk_close_socket(sd, l);
+        sd = JK_INVALID_SOCKET;
     }
     else {
         if (JK_IS_DEBUG_LEVEL(l))
             jk_log(l, JK_LOG_DEBUG, "socket %d connected to %s",
-                   sock, jk_dump_hinfo(addr, buf));
+                   sd, jk_dump_hinfo(addr, buf));
     }
     JK_TRACE_EXIT(l);
-    return sock;
+    return sd;
 }
 
-/** close the socket */
-
-int jk_close_socket(jk_sock_t s)
+/** Close the socket
+ * @param sd        socket to close
+ * @param l         logger
+ * @return          -1: some kind of error occured (!WIN32)
+ *                  SOCKET_ERROR: some kind of error occured  (WIN32)
+ *                  0: success
+ * @remark          Does not change errno
+ */
+int jk_close_socket(jk_sock_t sd, jk_logger_t *l)
 {
-#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-    if (s != INVALID_SOCKET)
-        return closesocket(s) ? -1 : 0;
-#else
-    if (s != -1)
-        return close(s);
-#endif
+    int rc;
+    int save_errno;
 
-    return -1;
+    JK_TRACE_ENTER(l);
+
+    if (!IS_VALID_SOCKET(sd)) {
+        JK_TRACE_EXIT(l);
+        return -1;
+    }
+
+    save_errno = errno;
+#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
+    rc = closesocket(sd) ? -1 : 0;
+#else
+    rc = close(sd);
+#endif
+    errno = save_errno;
+    JK_TRACE_EXIT(l);
+    return rc;
 }
 
 #ifndef MAX_SECS_TO_LINGER
-#define MAX_SECS_TO_LINGER 16
+#define MAX_SECS_TO_LINGER 30
 #endif
-#define SECONDS_TO_LINGER  1
+#define SECONDS_TO_LINGER  2
 
 #ifndef SHUT_WR
 #ifdef SD_SEND
@@ -484,122 +621,179 @@ int jk_close_socket(jk_sock_t s)
 #define SHUT_WR 0x01
 #endif
 #endif
-int jk_shutdown_socket(jk_sock_t s)
-{
-    unsigned char dummy[512];
-    int nbytes;
-    int ttl = 0;
-    int rc = 0;
-#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-    int tmout = SECONDS_TO_LINGER * 1000;
-    if (s == INVALID_SOCKET)
-#else
-    struct timeval tv;
-    if (s < 0)
-#endif
-        return -1;
 
+/** Drain and close the socket
+ * @param sd        socket to close
+ * @param l         logger
+ * @return          -1: socket to close is invalid
+ *                  -1: some kind of error occured (!WIN32)
+ *                  SOCKET_ERROR: some kind of error occured  (WIN32)
+ *                  0: success
+ * @remark          Does not change errno
+ */
+int jk_shutdown_socket(jk_sock_t sd, jk_logger_t *l)
+{
+    char dummy[512];
+    int rc = 0;
+    int save_errno;
+    fd_set rs;
+    struct timeval tv;
+    time_t start = time(NULL);
+
+    JK_TRACE_ENTER(l);
+
+    if (!IS_VALID_SOCKET(sd)) {
+        JK_TRACE_EXIT(l);
+        return -1;
+    }
+
+    save_errno = errno;
     /* Shut down the socket for write, which will send a FIN
      * to the peer.
      */
-    if (shutdown(s, SHUT_WR)) {
-        return jk_close_socket(s);
+    if (shutdown(sd, SHUT_WR)) {
+        rc = jk_close_socket(sd, l);
+        errno = save_errno;
+        JK_TRACE_EXIT(l);
+        return rc;
     }
-#if defined(WIN32)  || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-    if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO,
-                   (const char *) &tmout, sizeof(int)) == 0)
-        rc = 1;
-#elif defined(SO_RCVTIMEO) && defined(USE_SO_RCVTIMEO)
-    tv.tv_sec  = SECONDS_TO_LINGER;
-    tv.tv_usec = 0;
-    if (setsockopt(s, SOL_SOCKET, SO_RCVTIMEO,
-                   (const void *) &tv, sizeof(tv)))
-        rc = 1;
+
+    /* Set up to wait for readable data on socket... */
+    FD_ZERO(&rs);
+
+    do {
+        /* Read all data from the peer until we reach "end-of-file"
+         * (FIN from peer) or we've exceeded our overall timeout. If the
+         * backend does not send us bytes within 2 seconds
+         * (a value pulled from Apache 1.3 which seems to work well),
+         * close the connection.
+         */
+        FD_SET(sd, &rs);
+        tv.tv_sec  = SECONDS_TO_LINGER;
+        tv.tv_usec = 0;
+
+        if (select((int)sd + 1, &rs, NULL, NULL, &tv) > 0) {
+            do {
+#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
+                rc = recv(sd, &dummy[0], sizeof(dummy), 0);
+                if (JK_IS_SOCKET_ERROR(rc))
+                    JK_GET_SOCKET_ERRNO();
+#else
+                rc = read(sd, &dummy[0], sizeof(dummy));
 #endif
-    /* Read all data from the peer until we reach "end-of-file" (FIN
-     * from peer) or we've exceeded our overall timeout. If the client does
-     * not send us bytes within 12 second, close the connection.
-     */
-    while (rc) {
-        nbytes = jk_tcp_socket_recvfull(s, dummy, sizeof(dummy));
-        if (nbytes <= 0)
+            } while (JK_IS_SOCKET_ERROR(rc) && (errno == EINTR || errno == EAGAIN));
+
+            if (rc <= 0)
+                break;
+        }
+        else
             break;
-        ttl += SECONDS_TO_LINGER;
-        if (ttl > MAX_SECS_TO_LINGER)
-            break;
-    }
-    return jk_close_socket(s);
+
+    } while (difftime(time(NULL), start) < MAX_SECS_TO_LINGER);
+
+    rc = jk_close_socket(sd, l);
+    errno = save_errno;
+    JK_TRACE_EXIT(l);
+    return rc;
 }
 
-/** send a long message
- * @param sd  opened socket.
- * @param b   buffer containing the data.
- * @param len length to send.
- * @return    -2: send returned 0 ? what this that ?
- *            -3: send failed.
- *            >0: total size send.
+/** send a message
+ * @param sd  socket to use
+ * @param b   buffer containing the data
+ * @param len length to send
+ * @param l   logger
+ * @return    negative errno: write returns a fatal -1 (!WIN32)
+ *            negative pseudo errno: send returns SOCKET_ERROR (WIN32)
+ *            JK_SOCKET_EOF: no bytes could be sent
+ *            >0: success, provided number of bytes send
+ * @remark    Always closes socket in case of error
+ * @remark    Cares about errno
  * @bug       this fails on Unixes if len is too big for the underlying
- *             protocol.
+ *            protocol
  */
-int jk_tcp_socket_sendfull(jk_sock_t sd, const unsigned char *b, int len)
+int jk_tcp_socket_sendfull(jk_sock_t sd, const unsigned char *b, int len, jk_logger_t *l)
 {
     int sent = 0;
     int wr;
 
+    JK_TRACE_ENTER(l);
+
+    errno = 0;
     while (sent < len) {
         do {
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
             wr = send(sd, (const char*)(b + sent),
                       len - sent, 0);
-            if (wr == SOCKET_ERROR)
-                errno = WSAGetLastError() - WSABASEERR;
+            if (JK_IS_SOCKET_ERROR(wr))
+                JK_GET_SOCKET_ERRNO();
 #else
             wr = write(sd, b + sent, len - sent);
 #endif
-        } while (wr == -1 && (errno == EINTR || errno == EAGAIN));
+        } while (JK_IS_SOCKET_ERROR(wr) && (errno == EINTR || errno == EAGAIN));
 
-        if (wr == -1)
+        if (JK_IS_SOCKET_ERROR(wr)) {
+            jk_shutdown_socket(sd, l);
+            JK_TRACE_EXIT(l);
             return (errno > 0) ? -errno : errno;
-        else if (wr == 0)
+        }
+        else if (wr == 0) {
+            jk_shutdown_socket(sd, l);
+            JK_TRACE_EXIT(l);
             return JK_SOCKET_EOF;
+        }
         sent += wr;
     }
 
+    JK_TRACE_EXIT(l);
     return sent;
 }
 
-/** receive len bytes. Used in ajp_common.
- * @param sd  opened socket.
- * @param b   buffer to store the data.
- * @param len length to receive.
- * @return    <0: receive failed or connection closed.
- *            >0: length of the received data.
+/** receive a message
+ * @param sd  socket to use
+ * @param b   buffer to store the data
+ * @param len length to receive
+ * @param l   logger
+ * @return    negative errno: read returns a fatal -1 (!WIN32)
+ *            negative pseudo errno: recv returns SOCKET_ERROR (WIN32)
+ *            JK_SOCKET_EOF: no bytes could be read
+ *            >0: success, requested number of bytes received
+ * @remark    Always closes socket in case of error
+ * @remark    Cares about errno
  */
-int jk_tcp_socket_recvfull(jk_sock_t sd, unsigned char *b, int len)
+int jk_tcp_socket_recvfull(jk_sock_t sd, unsigned char *b, int len, jk_logger_t *l)
 {
     int rdlen = 0;
     int rd;
 
+    JK_TRACE_ENTER(l);
+
+    errno = 0;
     while (rdlen < len) {
         do {
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
             rd = recv(sd, (char *)b + rdlen,
                       len - rdlen, 0);
-            /* Assuming SOCKET_ERROR is -1 on NETWARE too */
-            if (rd == SOCKET_ERROR)
-                errno = WSAGetLastError() - WSABASEERR;
+            if (JK_IS_SOCKET_ERROR(rd))
+                JK_GET_SOCKET_ERRNO();
 #else
             rd = read(sd, (char *)b + rdlen, len - rdlen);
 #endif
-        } while (rd == -1 && (errno == EINTR || errno == EAGAIN));
+        } while (JK_IS_SOCKET_ERROR(rd) && (errno == EINTR || errno == EAGAIN));
 
-        if (rd == -1)
+        if (JK_IS_SOCKET_ERROR(rd)) {
+            jk_shutdown_socket(sd, l);
+            JK_TRACE_EXIT(l);
             return (errno > 0) ? -errno : errno;
-        else if (rd == 0)
+        }
+        else if (rd == 0) {
+            jk_shutdown_socket(sd, l);
+            JK_TRACE_EXIT(l);
             return JK_SOCKET_EOF;
+        }
         rdlen += rd;
     }
 
+    JK_TRACE_EXIT(l);
     return rdlen;
 }
 
@@ -619,56 +813,120 @@ char *jk_dump_hinfo(struct sockaddr_in *saddr, char *buf)
     return buf;
 }
 
+/** Wait for input event on socket until timeout
+ * @param sd      socket to use
+ * @param timeout wait timeout in milliseconds
+ * @param l       logger
+ * @return        JK_FALSE: Timeout expired without something to read
+ *                JK_FALSE: Error during waiting
+ *                JK_TRUE: success
+ * @remark        Does not close socket in case of error
+ *                to allow for iterative waiting
+ * @remark        Cares about errno
+ */
+int jk_is_input_event(jk_sock_t sd, int timeout, jk_logger_t *l)
+{
+    fd_set rset;
+    struct timeval tv;
+    int rc;
+    int save_errno;
+
+    JK_TRACE_ENTER(l);
+
+    errno = 0;
+    FD_ZERO(&rset);
+    FD_SET(sd, &rset);
+    tv.tv_sec = timeout / 1000;
+    tv.tv_usec = (timeout % 1000) * 1000;
+
+    do {
+        rc = select((int)sd + 1, &rset, NULL, NULL, &tv);
+    } while (rc < 0 && errno == EINTR);
+
+    if (rc == 0) {
+        /* Timeout. Set the errno to timeout */
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
-int jk_is_socket_connected(jk_sock_t sock)
+        errno = WSAETIMEDOUT - WSABASEERR;
+#else
+        errno = ETIMEDOUT;
+#endif
+        JK_TRACE_EXIT(l);
+        return JK_FALSE;
+    }
+    else if (rc < 0) {
+        save_errno = errno;
+        jk_log(l, JK_LOG_WARNING,
+               "error during select on socket sd = %d (errno=%d)", sd, errno);
+        errno = save_errno;
+        JK_TRACE_EXIT(l);
+        return JK_FALSE;
+    }
+    errno = 0;
+    JK_TRACE_EXIT(l);
+    return JK_TRUE;
+}
+
+/** Test if a socket is still connected
+ * @param sd   socket to use
+ * @param l    logger
+ * @return     JK_FALSE: failure
+ *             JK_TRUE: success
+ * @remark     Always closes socket in case of error
+ * @remark     Cares about errno
+ */
+int jk_is_socket_connected(jk_sock_t sd, jk_logger_t *l)
 {
     fd_set fd;
     struct timeval tv;
     int rc;
 
-    FD_ZERO(&fd);
-    FD_SET(sock, &fd);
-
-    /* Wait one microsecond */
-    tv.tv_sec  = 0;
-    tv.tv_usec = 1;
-
-    /* If we get a timeout, then we are still connected */
-    if ((rc = select(sock + 1, &fd, NULL, NULL, &tv)) == 0) {
-        errno = 0;
-        return 1;
-    }
-    else {
-        if (rc == SOCKET_ERROR)
-            errno = WSAGetLastError() - WSABASEERR;
-        else
-            errno = EOF;
-        return 0;
-    }
-}
-#else
-int jk_is_socket_connected(jk_sock_t sock)
-{
-    char test_buffer[1];
-    int  rd;
-    int  saved_errno;
+    JK_TRACE_ENTER(l);
 
     errno = 0;
-    /* Set socket to nonblocking */
-    if (sononblock(sock) != 0)
-        return 0;
+    FD_ZERO(&fd);
+    FD_SET(sd, &fd);
+
+    /* Initially test the socket without any blocking.
+     */
+    tv.tv_sec  = 0;
+    tv.tv_usec = 0;
+
     do {
-        rd = read(sock, test_buffer, 1);
-    } while (rd == -1 && errno == EINTR);
-    saved_errno = errno;
-    soblock(sock);
-    if (rd == -1 && saved_errno == EWOULDBLOCK) {
-        errno = 0;
-        return 1;
+        rc = select((int)sd + 1, &fd, NULL, NULL, &tv);
+        JK_GET_SOCKET_ERRNO();
+        /* Wait one microsecond on next select, if EINTR */
+        tv.tv_sec  = 0;
+        tv.tv_usec = 1;
+    } while (JK_IS_SOCKET_ERROR(rc) && errno == EINTR);
+
+    errno = 0;
+    if (rc == 0) {
+        /* If we get a timeout, then we are still connected */
+        JK_TRACE_EXIT(l);
+        return JK_TRUE;
     }
-    else {
-        errno = saved_errno ? saved_errno : EOF;
-        return 0;
-    }
-}
+    else if (rc == 1) {
+#if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
+        u_long nr;
+        rc = ioctlsocket(sd, FIONREAD, &nr);
+        if (rc == 0) {
+            if (WSAGetLastError() == 0)
+                errno = 0;
+            else
+                JK_GET_SOCKET_ERRNO();
+        }
+#else
+        int nr;
+        rc = ioctl(sd, FIONREAD, (void*)&nr);
 #endif
+        if (rc == 0 && nr != 0) {
+            JK_TRACE_EXIT(l);
+            return JK_TRUE;
+        }
+        JK_GET_SOCKET_ERRNO();
+    }
+    jk_shutdown_socket(sd, l);
+
+    JK_TRACE_EXIT(l);
+    return JK_FALSE;
+}
