@@ -19,19 +19,14 @@
  * Description: Global definitions and include files that should exist     *
  *              anywhere                                                   *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 538911 $                                               *
+ * Version:     $Revision: 604383 $                                               *
  ***************************************************************************/
 
 #ifndef JK_GLOBAL_H
 #define JK_GLOBAL_H
 
-#ifndef NETWARE
-#if !defined(WIN32) && !defined(AS400)
+#if !defined(WIN32) && !defined(AS400) && !defined(NETWARE)
 #include "portable.h"
-#else
-#define HAVE_VSNPRINTF
-#define HAVE_SNPRINTF
-#endif
 #endif
 
 #if defined(WIN32)
@@ -62,8 +57,8 @@
 #define _CRT_SECURE_NO_DEPRECATE
 #endif
 #pragma warning(disable: 4996)
-#endif
-#endif
+#endif /* defined(_MSC_VER) && _MSC_VER >= 1400 */
+#endif /* defined(WIN32) */
 
 #include "jk_version.h"
 
@@ -113,7 +108,7 @@ extern char *strdup(const char *str);
 #define NOIME
 #endif
 #include <windows.h>
-/* 
+/*
  * Add a _very_few_ declarations missing from the restricted set of headers
  * (If this list becomes extensive, re-enable the required headers above!)
  * winsock headers were excluded by WIN32_LEAN_AND_MEAN, so include them now
@@ -121,10 +116,10 @@ extern char *strdup(const char *str);
 #include <winsock2.h>
 #include <mswsock.h>
 #include <ws2tcpip.h>
-#endif
+#endif /* _WINDOWS_ */
 #include <sys/timeb.h>
 #include <process.h>
-#else
+#else /* WIN32 */
 #include <unistd.h>
 #if defined(NETWARE) && defined(__NOVELL_LIBC__)
 #include "novsock2.h"
@@ -148,11 +143,11 @@ extern char *strdup(const char *str);
 #if !defined(HPUX11) && !defined(AS400)
 #include <sys/select.h>
 #endif
-#endif
+#endif /* NETWARE */
 
 #include <sys/time.h>
 #include <sys/ioctl.h>
-#endif
+#endif /* WIN32 */
 
 #ifdef __cplusplus
 extern "C"
@@ -184,11 +179,19 @@ extern "C"
 #define JK_PATH_SESSION_IDENTIFIER ";jsessionid"
 
 #if defined(WIN32) || defined(NETWARE)
+#ifdef __GNUC__
+#define JK_METHOD
+#define C_LEVEL_TRY_START
+#define C_LEVEL_TRY_END
+#define C_LEVEL_FINALLY_START
+#define C_LEVEL_FINALLY_END
+#else
 #define JK_METHOD __stdcall
 #define C_LEVEL_TRY_START       __try {
 #define C_LEVEL_TRY_END         }
 #define C_LEVEL_FINALLY_START   __finally {
 #define C_LEVEL_FINALLY_END     }
+#endif
 #define PATH_SEPERATOR          (';')
 #define FILE_SEPERATOR          ('\\')
 #define PATH_ENV_VARIABLE       ("PATH")
@@ -197,7 +200,7 @@ extern "C"
 #ifndef strcasecmp
 #define strcasecmp stricmp
 #endif
-#else
+#else /* defined(WIN32) || defined(NETWARE) */
 #define JK_METHOD
 #define C_LEVEL_TRY_START
 #define C_LEVEL_TRY_END
@@ -206,19 +209,19 @@ extern "C"
 #define PATH_SEPERATOR          (':')
 #define FILE_SEPERATOR          ('/')
 #define PATH_ENV_VARIABLE       ("LD_LIBRARY_PATH")
-#endif
+#endif /* defined(WIN32) || defined(NETWARE) */
 
 /* HTTP Error codes
  */
 
-#define JK_HTTP_OK               200
-#define JK_HTTP_BAD_REQUEST      400
-#define JK_REQUEST_TOO_LARGE     413
-#define JK_HTTP_SERVER_ERROR     500
-#define JK_HTTP_NOT_IMPLEMENTED  501
-#define JK_HTTP_BAD_GATEWAY      502
-#define JK_HTTP_SERVER_BUSY      503
-#define JK_HTTP_GATEWAY_TIME_OUT 504
+#define JK_HTTP_OK                200
+#define JK_HTTP_BAD_REQUEST       400
+#define JK_HTTP_REQUEST_TOO_LARGE 413
+#define JK_HTTP_SERVER_ERROR      500
+#define JK_HTTP_NOT_IMPLEMENTED   501
+#define JK_HTTP_BAD_GATEWAY       502
+#define JK_HTTP_SERVER_BUSY       503
+#define JK_HTTP_GATEWAY_TIME_OUT  504
 
 
 /*
@@ -233,15 +236,14 @@ extern "C"
  * JK options
  */
 
-#define JK_OPT_FWDURIMASK           0x0003
+#define JK_OPT_FWDURIMASK           0x0007
 
 #define JK_OPT_FWDURICOMPAT         0x0001
 #define JK_OPT_FWDURICOMPATUNPARSED 0x0002
 #define JK_OPT_FWDURIESCAPED        0x0003
+#define JK_OPT_FWDURIPROXY          0x0004
 
-#define JK_OPT_FWDURIDEFAULT        JK_OPT_FWDURICOMPATUNPARSED
-
-#define JK_OPT_FWDKEYSIZE           0x0004
+#define JK_OPT_FWDURIDEFAULT        JK_OPT_FWDURIPROXY
 
 #define JK_OPT_FWDDIRS              0x0008
 /* Forward local instead remote address */
@@ -250,6 +252,8 @@ extern "C"
 #define JK_OPT_FLUSHEADER           0x0040
 #define JK_OPT_DISABLEREUSE         0x0080
 #define JK_OPT_FWDCERTCHAIN         0x0100
+#define JK_OPT_FWDKEYSIZE           0x0200
+#define JK_OPT_REJECTUNSAFE         0x0400
 
 /* Check for EBCDIC systems */
 
@@ -282,6 +286,13 @@ extern "C"
 
 #endif                          /* APR_CHARSET_EBCDIC */
 
+/* on i5/OS V5R4 HTTP/APR APIs and Datas are in UTF */
+#if defined(AS400_UTF8)
+#undef USE_CHARSET_EBCDIC
+#define jk_xlate_to_ascii(b, l) /* NOOP */
+#define jk_xlate_from_ascii(b, l)       /* NOOP */
+#endif
+
 /* jk_uint32_t defines a four byte word */
 /* jk_uint64_t defines a eight byte word */
 #if defined (WIN32)
@@ -291,6 +302,7 @@ extern "C"
     typedef unsigned __int64 jk_uint64_t;
 #define JK_UINT64_T_FMT "I64u"
 #define JK_UINT64_T_HEX_FMT "I64x"
+#define JK_PID_T_FMT "d"
 #elif defined(AS400) || defined(NETWARE)
     typedef unsigned int jk_uint32_t;
 #define JK_UINT32_T_FMT "u"
@@ -298,6 +310,7 @@ extern "C"
     typedef unsigned long long jk_uint64_t;
 #define JK_UINT64_T_FMT "llu"
 #define JK_UINT64_T_HEX_FMT "llx"
+#define JK_PID_T_FMT "d"
 #else
 #include "jk_types.h"
 #endif
@@ -309,7 +322,11 @@ extern "C"
 
 #ifdef WIN32
 /* For WIN32, emulate gettimeofday() using _ftime() */
-#define gettimeofday(tv,tz) { struct _timeb tb; _ftime(&tb); (tv)->tv_sec = tb.time; (tv)->tv_usec = tb.millitm * 1000; }
+#define gettimeofday(tv, tz) { struct _timeb tb; _ftime(&tb); \
+                               (tv)->tv_sec = (long)tb.time;  \
+                               (tv)->tv_usec = tb.millitm * 1000; }
+#define HAVE_VSNPRINTF
+#define HAVE_SNPRINTF
 #ifdef HAVE_APR
 #define snprintf apr_snprintf
 #define vsnprintf apr_vsnprintf
@@ -318,7 +335,7 @@ extern "C"
 #define snprintf _snprintf
 #define vsnprintf _vsnprintf
 #endif
-#endif
+#endif /* WIN32" */
 
 /* Use apr snprintf() and vsnprintf() when needed */
 #if defined(HAVE_APR)
@@ -330,11 +347,14 @@ extern "C"
 #endif
 #endif
 
-/* XXXX There is a snprintf() and vsnprintf() in jk_util.c */
-/* if those work remove the #define. */
-#if defined(NETWARE) || defined(AS400)
-#define USE_SPRINTF
-#define USE_VSPRINTF
+/* Use ap snprintf() and vsnprintf() when needed */
+#if !defined(HAVE_APR)
+#if !defined(HAVE_SNPRINTF)
+#define snprintf ap_snprintf
+#endif
+#if !defined(HAVE_VSNPRINTF)
+#define vsnprintf ap_vsnprintf
+#endif
 #endif
 
 #if defined(WIN32) || (defined(NETWARE) && defined(__NOVELL_LIBC__))
@@ -347,6 +367,10 @@ typedef int jk_sock_t;
 #define JK_INVALID_SOCKET  (-1)
 #endif
 
+#ifdef AS400_UTF8
+#define strcasecmp(a,b) apr_strnatcasecmp(a,b)
+#endif
+
 #ifdef __cplusplus
 }
 #endif                          /* __cplusplus */
