@@ -19,7 +19,7 @@
  * Description: load balance worker header file                            *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Rainer Jung <rjung@apache.org>                             *
- * Version:     $Revision: 500537 $                                           *
+ * Version:     $Revision: 550272 $                                           *
  ***************************************************************************/
 
 #ifndef JK_LB_WORKER_H
@@ -57,23 +57,23 @@ extern "C"
 #define JK_LB_LOCK_TEXT_OPTIMISTIC     ("Optimistic")
 #define JK_LB_LOCK_TEXT_PESSIMISTIC    ("Pessimistic")
 #define JK_LB_LOCK_TEXT_DEF            (JK_LB_LOCK_TEXT_OPTIMISTIC)
-#define JK_LB_STATE_NA                 (0)
+#define JK_LB_STATE_IDLE               (0)
 #define JK_LB_STATE_OK                 (1)
 #define JK_LB_STATE_RECOVER            (2)
 #define JK_LB_STATE_BUSY               (3)
 #define JK_LB_STATE_ERROR              (4)
 #define JK_LB_STATE_FORCE              (5)
 #define JK_LB_STATE_PROBE              (6)
-#define JK_LB_STATE_DEF                (JK_LB_STATE_NA)
-#define JK_LB_STATE_TEXT_NA            ("N/A")
+#define JK_LB_STATE_DEF                (JK_LB_STATE_IDLE)
+#define JK_LB_STATE_TEXT_IDLE          ("OK/IDLE")
 #define JK_LB_STATE_TEXT_OK            ("OK")
-#define JK_LB_STATE_TEXT_RECOVER       ("REC")
-#define JK_LB_STATE_TEXT_BUSY          ("BSY")
+#define JK_LB_STATE_TEXT_RECOVER       ("ERR/REC")
+#define JK_LB_STATE_TEXT_BUSY          ("OK/BUSY")
 #define JK_LB_STATE_TEXT_ERROR         ("ERR")
-#define JK_LB_STATE_TEXT_FORCE         ("FRC")
-#define JK_LB_STATE_TEXT_PROBE         ("PRB")
+#define JK_LB_STATE_TEXT_FORCE         ("ERR/FRC")
+#define JK_LB_STATE_TEXT_PROBE         ("ERR/PRB")
 #define JK_LB_STATE_TEXT_MAX           (JK_LB_STATE_PROBE)
-#define JK_LB_STATE_TEXT_DEF           (JK_LB_STATE_TEXT_NA)
+#define JK_LB_STATE_TEXT_DEF           (JK_LB_STATE_TEXT_IDLE)
 #define JK_LB_ACTIVATION_ACTIVE        (0)
 #define JK_LB_ACTIVATION_DISABLED      (1)
 #define JK_LB_ACTIVATION_STOPPED       (2)
@@ -136,6 +136,7 @@ struct lb_worker
     int          sticky_session;
     int          sticky_session_force;
     int          recover_wait_time;
+    int          max_reply_timeouts;
     int          retries;
     int          lbmethod;
     int          lblock;
