@@ -19,9 +19,9 @@
  * Description: General purpose map object                                 *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 521770 $                                          *
+ * Version:     $Revision: 591502 $                                          *
  ***************************************************************************/
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
 #include "apr_xlate.h"
 #endif
 
@@ -249,7 +249,7 @@ char **jk_map_get_string_list(jk_map_t *m,
     const char *l = jk_map_get_string(m, name, def);
     char **ar = NULL;
 
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
     char *lasts;
 #endif
 
@@ -269,7 +269,7 @@ char **jk_map_get_string_list(jk_map_t *m,
          * GS, in addition to VG's patch, we now need to
          * strtok also by a "*"
          */
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
         for (p = strtok_r(v, " \t,", &lasts);
              p; p = strtok_r(NULL, " \t,", &lasts))
 #else
@@ -305,10 +305,10 @@ int jk_map_get_int_list(jk_map_t *m,
 {
     const char *l = jk_map_get_string(m, name, def);
 
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
     char *lasts;
 #endif
-    
+
     if (!list_len)
         return 0;
 
@@ -326,7 +326,7 @@ int jk_map_get_int_list(jk_map_t *m,
          * GS, in addition to VG's patch, we now need to
          * strtok also by a "*"
          */
-#if defined(AS400) || defined(_REENTRANT)
+#ifdef _REENTRANT
         for (p = strtok_r(v, " \t,", &lasts);
              p; p = strtok_r(NULL, " \t,", &lasts))
 #else
@@ -339,7 +339,7 @@ int jk_map_get_int_list(jk_map_t *m,
                 index++;
             }
             else
-                break;                
+                break;
         }
         return index;
     }
@@ -395,7 +395,67 @@ int jk_map_put(jk_map_t *m, const char *name, const void *value, void **old)
     return rc;
 }
 
-int jk_map_read_property(jk_map_t *m, const char *str, int allow_duplicates, jk_logger_t *l)
+
+static int jk_map_validate_property(char *prp, jk_logger_t *l)
+{
+    /* check the worker properties */
+    if (!jk_is_valid_property(prp)) {
+        jk_log(l, JK_LOG_ERROR,
+               "The attribute '%s' is not supported - please check"
+               " the documentation for the supported attributes.",
+               prp);
+        return JK_FALSE;
+    }
+    if (jk_is_deprecated_property(prp)) {
+        jk_log(l, JK_LOG_WARNING,
+               "The attribute '%s' is deprecated - please check"
+               " the documentation for the correct replacement.",
+               prp);
+    }
+    return JK_TRUE;
+}
+
+static int jk_map_handle_duplicates(jk_map_t *m, const char *prp, char **v,
+                                    int treatment, jk_logger_t *l)
+{
+    const char *oldv = jk_map_get_string(m, prp, NULL);
+    if (oldv) {
+        if ((treatment == JK_MAP_HANDLE_DUPLICATES)
+            && jk_is_unique_property(prp) == JK_FALSE) {
+            char *tmpv = jk_pool_alloc(&m->p,
+                                       strlen(*v) + strlen(oldv) + 3);
+            if (tmpv) {
+                char sep = '*';
+                if (jk_is_path_property(prp))
+                    sep = PATH_SEPERATOR;
+                else if (jk_is_cmd_line_property(prp))
+                    sep = ' ';
+                else if (jk_is_list_property(prp))
+                    sep = ',';
+                sprintf(tmpv, "%s%c%s", oldv, sep, *v);
+            }
+            *v = tmpv;
+            if (JK_IS_DEBUG_LEVEL(l))
+                jk_log(l, JK_LOG_DEBUG,
+                       "Concatenated value is: %s -> %s",
+                       prp, *v);
+            return JK_FALSE;
+        }
+        else {
+            jk_log(l, JK_LOG_WARNING,
+                   "Duplicate key '%s' detected - previous value '%s'"
+                   " will be overwritten with '%s'.",
+                   prp, oldv ? oldv : "(null)", v ? *v : "(null)");
+            return JK_TRUE;
+        }
+    }
+    else {
+        return JK_TRUE;
+    }
+}
+
+int jk_map_read_property(jk_map_t *m, const char *str,
+                         int treatment, jk_logger_t *l)
 {
     int rc = JK_TRUE;
     char buf[LENGTH_OF_LINE + 1];
@@ -417,54 +477,21 @@ int jk_map_read_property(jk_map_t *m, const char *str, int allow_duplicates, jk_
             trim(prp);
             trim(v);
             if (strlen(v) && strlen(prp)) {
-                const char *oldv;
-                int off = (int)strlen(prp) - (int)JK_MAP_REFERENCE_SZ;
-                /* check the worker properties */
-                if (off <= 0 || strncmp(&prp[off], JK_MAP_REFERENCE, JK_MAP_REFERENCE_SZ) ) {
-                    if (!jk_is_valid_property(prp)) {
-                        jk_log(l, JK_LOG_ERROR,
-                               "The attribute '%s' is not supported - please check"
-                               " the documentation for the supported attributes.",
-                               prp);
-                        return JK_FALSE;
-                    }
-                    if (jk_is_deprecated_property(prp)) {
-                        jk_log(l, JK_LOG_WARNING,
-                               "The attribute '%s' is deprecated - please check"
-                               " the documentation for the correct replacement.",
-                               prp);
-                    }
-                }
-                oldv = jk_map_get_string(m, prp, NULL);
-                v = jk_map_replace_properties(m, v);
-                if (oldv) {
-                    if (allow_duplicates && jk_is_unique_property(prp) == JK_FALSE) {
-                        char *tmpv = jk_pool_alloc(&m->p,
-                                           strlen(v) + strlen(oldv) + 3);
-                        if (tmpv) {
-                            char sep = '*';
-                            if (jk_is_path_property(prp))
-                                sep = PATH_SEPERATOR;
-                            else if (jk_is_cmd_line_property(prp))
-                                sep = ' ';
-                            else if (jk_is_list_property(prp))
-                                sep = ',';
-                            sprintf(tmpv, "%s%c%s", oldv, sep, v);
-                        }
-                        v = tmpv;
-                    }
-                    else {
-                        jk_log(l, JK_LOG_WARNING,
-                               "Duplicate key '%s' detected - previous value '%s'"
-                               " will be overwritten with '%s'.",
-                               prp, oldv ? oldv : "(null)", v ? v : "(null)");
-                        v = jk_pool_strdup(&m->p, v);
-                    }
-                }
-                else {
+                if (treatment == JK_MAP_HANDLE_RAW) {
                     v = jk_pool_strdup(&m->p, v);
                 }
+                else {
+                    if (jk_map_validate_property(prp, l) == JK_FALSE)
+                        return JK_FALSE;
+                    v = jk_map_replace_properties(m, v);
+                    if (jk_map_handle_duplicates(m, prp, &v, treatment, l) == JK_TRUE)
+                        v = jk_pool_strdup(&m->p, v);
+                }
                 if (v) {
+                    if (JK_IS_DEBUG_LEVEL(l))
+                        jk_log(l, JK_LOG_DEBUG,
+                               "Adding property '%s' with value '%s' to map.",
+                               prp, v);
                     jk_map_put(m, prp, v, NULL);
                 }
                 else {
@@ -478,16 +505,17 @@ int jk_map_read_property(jk_map_t *m, const char *str, int allow_duplicates, jk_
 }
 
 
-int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified, int allow_duplicates, jk_logger_t *l)
+int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified,
+                           int treatment, jk_logger_t *l)
 {
     int rc = JK_FALSE;
 
     if (m && f) {
         struct stat statbuf;
         FILE *fp;
-        if ((rc = stat(f, &statbuf)) == -1)
+        if (jk_stat(f, &statbuf) == -1)
             return JK_FALSE;
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
         fp = fopen(f, "r, o_ccsid=0");
 #else
         fp = fopen(f, "r");
@@ -502,7 +530,7 @@ int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified, int all
             while (NULL != (prp = fgets(buf, LENGTH_OF_LINE, fp))) {
                 trim_prp_comment(prp);
                 if (*prp) {
-                    if ((rc = jk_map_read_property(m, prp, allow_duplicates, l)) == JK_FALSE)
+                    if ((rc = jk_map_read_property(m, prp, treatment, l)) == JK_FALSE)
                         break;
                 }
             }
@@ -543,9 +571,52 @@ void *jk_map_value_at(jk_map_t *m, int idex)
     return NULL;
 }
 
+void jk_map_dump(jk_map_t *m, jk_logger_t *l)
+{
+    if (m) {
+        int s = jk_map_size(m);
+        int i;
+        for (i=0;i<s;i++) {
+            if (!jk_map_name_at(m, i)) {
+                jk_log(l, JK_LOG_WARNING,
+                       "Map contains empty name at index %d\n", i);
+            }
+            if (!jk_map_value_at(m, i)) {
+                jk_log(l, JK_LOG_WARNING,
+                       "Map contains empty value for name '%s' at index %d\n",
+                       jk_map_name_at(m, i), i);
+            }
+            if (JK_IS_DEBUG_LEVEL(l)) {
+                jk_log(l, JK_LOG_DEBUG,
+                       "Dump of map: '%s' -> '%s'",
+                       jk_map_name_at(m, i) ? jk_map_name_at(m, i) : "(null)",
+                       jk_map_value_at(m, i) ? jk_map_value_at(m, i) : "(null)");
+            }
+        }
+    }
+}
+
+int jk_map_copy(jk_map_t *src, jk_map_t *dst)
+{
+    int sz = jk_map_size(src);
+    int i;
+    for (i = 0; i < sz; i++) {
+        const char *name = jk_map_name_at(src, i);
+        if (jk_map_get(dst, name, NULL) == NULL) {
+            if (!jk_map_put(dst, name,
+                            jk_pool_strdup(&dst->p, jk_map_get_string(src, name, NULL)),
+                            NULL)) {
+                return JK_FALSE;
+            }
+        }
+    }
+    return JK_TRUE;
+}
+
+
 static void trim_prp_comment(char *prp)
 {
-#ifdef AS400
+#if defined(AS400) && !defined(AS400_UTF8)
     char *comment;
     /* lots of lines that translate a '#' realtime deleted   */
     comment = strchr(prp, *APR_NUMBERSIGN);
@@ -700,14 +771,16 @@ int jk_map_resolve_references(jk_map_t *m, const char *prefix,
                        "Checking for references with prefix %s with%s wildcard (recursion %d)",
                        prefix, wildcard? "" : "out", depth);
             for (i = 0; i < m->size; i++) {
-                if (m->values[i] && !strncmp(m->names[i], prefix, prelen)) {
+                char *v = (char *)m->values[i];
+                if (v && *v &&
+                    !strncmp(m->names[i], prefix, prelen)) {
                     size_t remain = strlen(m->names[i]) - prelen;
                     if ((remain == JK_MAP_REFERENCE_SZ ) || (wildcard && remain > JK_MAP_REFERENCE_SZ)) {
                         remain = strlen(m->names[i]) - JK_MAP_REFERENCE_SZ;
                         if (!strncmp(m->names[i] + remain, JK_MAP_REFERENCE, JK_MAP_REFERENCE_SZ)) {
                             char *from = jk_pool_alloc(&m->p,
                                                        (sizeof(char) *
-                                                       (strlen(m->values[i]) + 2)));
+                                                       (strlen(v) + 2)));
                             char *to = jk_pool_alloc(&m->p,
                                                      (sizeof(char) *
                                                      (remain + 2)));
@@ -717,14 +790,14 @@ int jk_map_resolve_references(jk_map_t *m, const char *prefix,
                                 rc = JK_FALSE;
                                 break;
                             }
-                            strcpy(from, m->values[i]);
-                            *(from+strlen(m->values[i]))   = '.';
-                            *(from+strlen(m->values[i])+1) = '\0';
+                            strcpy(from, v);
+                            *(from+strlen(v))   = '.';
+                            *(from+strlen(v)+1) = '\0';
                             strncpy(to, m->names[i], remain);
                             *(to+remain)   = '.';
                             *(to+remain+1) = '\0';
 
-                            rc = jk_map_resolve_references(m, m->values[i], 0, depth+1, l);
+                            rc = jk_map_resolve_references(m, v, 0, depth+1, l);
                             if (rc == JK_FALSE) {
                                 break;
                             }
@@ -736,7 +809,7 @@ int jk_map_resolve_references(jk_map_t *m, const char *prefix,
                             if (rc == JK_FALSE) {
                                 break;
                             }
-                            m->values[i] = NULL;
+                            *v = '\0';
                         }
                     }
                 }
@@ -806,78 +879,3 @@ int jk_map_inherit_properties(jk_map_t *m, const char *from, const char *to, jk_
     }
     return rc;
 }
-
-int jk_map_load_property(jk_map_t *m, const char *str, jk_logger_t *l)
-{
-    int rc = JK_TRUE;
-    char buf[LENGTH_OF_LINE + 1];
-    char *prp = &buf[0];
-
-    if (strlen(str) > LENGTH_OF_LINE) {
-        jk_log(l, JK_LOG_WARNING,
-               "Line to long (%d > %d), ignoring entry",
-               strlen(str), LENGTH_OF_LINE);
-        return JK_FALSE;
-    }
-
-    strcpy(prp, str);
-    if (trim(prp)) {
-        char *v = strchr(prp, '=');
-        if (v) {
-            *v = '\0';
-            v++;
-            trim(prp);
-            trim(v);
-            if (strlen(v) && strlen(prp)) {
-                v = jk_pool_strdup(&m->p, v);
-                if (v) {
-                    jk_map_put(m, prp, v, NULL);
-                }
-                else {
-                    JK_LOG_NULL_PARAMS(l);
-                    rc = JK_FALSE;
-                }
-            }
-        }
-    }
-    return rc;
-}
-
-
-int jk_map_load_properties(jk_map_t *m, const char *f, time_t *modified, jk_logger_t *l)
-{
-    int rc = JK_FALSE;
-
-    if (m && f) {
-        struct stat statbuf;
-        FILE *fp;
-        if ((rc = stat(f, &statbuf)) == -1)
-            return JK_FALSE;
-#ifdef AS400
-        fp = fopen(f, "r, o_ccsid=0");
-#else
-        fp = fopen(f, "r");
-#endif
-
-        if (fp) {
-            char buf[LENGTH_OF_LINE + 1];
-            char *prp;
-
-            rc = JK_TRUE;
-
-            while (NULL != (prp = fgets(buf, LENGTH_OF_LINE, fp))) {
-                trim_prp_comment(prp);
-                if (*prp) {
-                    if ((rc = jk_map_load_property(m, prp, l)) == JK_FALSE)
-                        break;
-                }
-            }
-            fclose(fp);
-            if (modified)
-                *modified = statbuf.st_mtime;
-        }
-    }
-
-    return rc;
-}
-
