@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -17,7 +18,7 @@
 /***************************************************************************
  * Description: Map object header file                                     *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 300292 $                                           *
+ * Version:     $Revision: 591502 $                                           *
  ***************************************************************************/
 
 #ifndef JK_MAP_H
@@ -29,6 +30,9 @@ extern "C"
 {
 #endif                          /* __cplusplus */
 
+#define JK_MAP_HANDLE_NORMAL     0
+#define JK_MAP_HANDLE_DUPLICATES 1
+#define JK_MAP_HANDLE_RAW        2
 
 struct jk_map;
 typedef struct jk_map jk_map_t;
@@ -57,11 +61,18 @@ char **jk_map_get_string_list(jk_map_t *m,
                            const char *name,
                            unsigned *list_len, const char *def);
 
+int jk_map_get_int_list(jk_map_t *m,
+                        const char *name,
+                        int *list, unsigned int list_len,
+                        const char *def);
+
+int jk_map_add(jk_map_t *m, const char *name, const void *value);
+
 int jk_map_put(jk_map_t *m, const char *name, const void *value, void **old);
 
-int jk_map_read_property(jk_map_t *m, const char *str);
+int jk_map_read_property(jk_map_t *m, const char *str, int treatment, jk_logger_t *l);
 
-int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified);
+int jk_map_read_properties(jk_map_t *m, const char *f, time_t *modified, int treatment, jk_logger_t *l);
 
 int jk_map_size(jk_map_t *m);
 
@@ -69,12 +80,19 @@ const char *jk_map_name_at(jk_map_t *m, int idex);
 
 void *jk_map_value_at(jk_map_t *m, int idex);
 
+void jk_map_dump(jk_map_t *m, jk_logger_t *l);
+
+int jk_map_copy(jk_map_t *src, jk_map_t *dst);
+
 /**
  *  Replace $(property) in value.
- * 
+ *
  */
-char *jk_map_replace_properties(const char *value, jk_map_t *m);
+char *jk_map_replace_properties(jk_map_t *m, const char *value);
 
+int jk_map_resolve_references(jk_map_t *m, const char *prefix, int wildcard, int depth, jk_logger_t *l);
+
+int jk_map_inherit_properties(jk_map_t *m, const char *from, const char *to, jk_logger_t *l);
 
 #ifdef __cplusplus
 }
