@@ -22,7 +22,7 @@
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Dan Milstein <danmil@shore.net>                            *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 483456 $                                          *
+ * Version:     $Revision: 604384 $                                          *
  ***************************************************************************/
 
 #ifndef JK_SERVICE_H
@@ -133,11 +133,11 @@ struct jk_ws_service
     const char *server_name;
     unsigned server_port;
     char *server_software;
-    unsigned content_length;        /* integer that represents the content  */
+    jk_uint64_t content_length; /* 64 bit integer that represents the content */
     /* length should be 0 if unknown.        */
     unsigned is_chunked;    /* 1 if content length is unknown (chunked rq) */
     unsigned no_more_chunks;        /* 1 if last chunk has been read */
-    unsigned content_read;  /* number of bytes read */
+    jk_uint64_t content_read;  /* number of bytes read */
 
     /*
      * SSL information
@@ -222,6 +222,11 @@ struct jk_ws_service
      */
     int flush_header;
 
+    /*
+     * JK_TRUE if response headers have been sent back
+     */
+    int response_started;
+
     /*
      * HTTP status sent from container.
      */
@@ -278,6 +283,21 @@ struct jk_ws_service
                                       const char *const *log_names,
                                       const char *const *log_values,
                                       unsigned num_of_items);
+
+    /*
+     * Iterate through all vhosts
+     */
+    void *(JK_METHOD * next_vhost) (void *d);
+
+    /*
+     * String representation of a vhost
+     */
+    void (JK_METHOD * vhost_to_text) (void *d, char *buf, size_t len);
+
+    /*
+     * Get uw_map associated with a vhost
+     */
+    jk_uri_worker_map_t *(JK_METHOD * vhost_to_uw_map) (void *d);
 };
 
 /*
@@ -312,8 +332,16 @@ struct jk_ws_service
  */
 struct jk_endpoint
 {
-    size_t rd;
-    size_t wr;
+    jk_uint64_t rd;
+    jk_uint64_t wr;
+
+    /*
+     * Flag to pass back recoverability status from
+     * a load balancer member to the load balancer itself.
+     * Depending on the configuration and request status
+     * recovery is not allowed.
+     */
+    int recoverable;
 
     /*
      * A 'this' pointer which is used by the subclasses of this class to
