@@ -20,7 +20,7 @@
  *                                                                         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 562174 $                                          *
+ * Version:     $Revision: 605846 $                                          *
  ***************************************************************************/
 
 #include "jk_pool.h"
@@ -176,16 +176,18 @@ static int wildchar_match(const char *str, const char *exp, int icase)
     return (str[x] != '\0');
 }
 
-int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map,
+int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map_p,
                          jk_map_t *init_data, jk_logger_t *l)
 {
     JK_TRACE_ENTER(l);
 
-    if (uw_map) {
+    if (uw_map_p) {
         int rc;
-        *uw_map = (jk_uri_worker_map_t *)calloc(1, sizeof(jk_uri_worker_map_t));
+        jk_uri_worker_map_t *uw_map;
+        *uw_map_p = (jk_uri_worker_map_t *)calloc(1, sizeof(jk_uri_worker_map_t));
+        uw_map = *uw_map_p;
 
-        JK_INIT_CS(&((*uw_map)->cs), rc);
+        JK_INIT_CS(&(uw_map->cs), rc);
         if (rc == JK_FALSE) {
             jk_log(l, JK_LOG_ERROR,
                    "creating thread lock (errno=%d)",
@@ -193,8 +195,21 @@ int uri_worker_map_alloc(jk_uri_worker_map_t **uw_map,
             JK_TRACE_EXIT(l);
             return JK_FALSE;
         }
+
+        jk_open_pool(&(uw_map->p),
+                     uw_map->buf, sizeof(jk_pool_atom_t) * BIG_POOL_SIZE);
+        uw_map->size = 0;
+        uw_map->nosize = 0;
+        uw_map->capacity = 0;
+        uw_map->maps = NULL;
+        uw_map->fname = NULL;
+        uw_map->reject_unsafe = 0;
+        uw_map->reload = JK_URIMAP_DEF_RELOAD;
+        uw_map->modified = 0;
+        uw_map->checked = 0;
+
         if (init_data)
-            rc = uri_worker_map_open(*uw_map, init_data, l);
+            rc = uri_worker_map_open(uw_map, init_data, l);
         JK_TRACE_EXIT(l);
         return rc;
     }
@@ -406,19 +421,8 @@ int uri_worker_map_open(jk_uri_worker_map_t *uw_map,
 
     JK_TRACE_ENTER(l);
 
-    uw_map->size = 0;
-    uw_map->capacity = 0;
-
     if (uw_map) {
-        int sz;
-
-        rc = JK_TRUE;
-        jk_open_pool(&uw_map->p,
-                     uw_map->buf, sizeof(jk_pool_atom_t) * SMALL_POOL_SIZE);
-        uw_map->size = 0;
-        uw_map->maps = NULL;
-
-        sz = jk_map_size(init_data);
+        int sz = jk_map_size(init_data);
 
         jk_log(l, JK_LOG_DEBUG,
                "rule map size is %d",
@@ -480,9 +484,61 @@ int uri_worker_map_open(jk_uri_worker_map_t *uw_map,
     return rc;
 }
 
-static int is_nomap_match(jk_uri_worker_map_t *uw_map,
-                          const char *uri, const char* worker,
-                          jk_logger_t *l)
+static const char *find_match(jk_uri_worker_map_t *uw_map,
+                        const char *url, jk_logger_t *l)
+{
+    unsigned int i;
+
+    JK_TRACE_ENTER(l);
+
+    for (i = 0; i < uw_map->size; i++) {
+        uri_worker_record_t *uwr = uw_map->maps[i];
+
+        /* Check for match types */
+        if ((uwr->match_type & MATCH_TYPE_DISABLED) ||
+            (uwr->match_type & MATCH_TYPE_NO_MATCH))
+            continue;
+
+        if (JK_IS_DEBUG_LEVEL(l))
+            jk_log(l, JK_LOG_DEBUG, "Attempting to map context URI '%s=%s' source '%s'",
+                   uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
+
+        if (uwr->match_type & MATCH_TYPE_WILDCHAR_PATH) {
+            /* Map is already sorted by context_len */
+            if (wildchar_match(url, uwr->context,
+#ifdef WIN32
+                               0
+#else
+                               0
+#endif
+                               ) == 0) {
+                    if (JK_IS_DEBUG_LEVEL(l))
+                        jk_log(l, JK_LOG_DEBUG,
+                               "Found a wildchar match '%s=%s'",
+                               uwr->context, uwr->worker_name);
+                    JK_TRACE_EXIT(l);
+                    return uwr->worker_name;
+             }
+        }
+        else if (JK_STRNCMP(uwr->context, url, uwr->context_len) == 0) {
+            if (strlen(url) == uwr->context_len) {
+                if (JK_IS_DEBUG_LEVEL(l))
+                    jk_log(l, JK_LOG_DEBUG,
+                           "Found an exact match '%s=%s'",
+                           uwr->context, uwr->worker_name);
+                JK_TRACE_EXIT(l);
+                return uwr->worker_name;
+            }
+        }
+    }
+
+    JK_TRACE_EXIT(l);
+    return NULL;
+}
+
+static int is_nomatch(jk_uri_worker_map_t *uw_map,
+                      const char *uri, const char* worker,
+                      jk_logger_t *l)
 {
     unsigned int i;
 
@@ -496,7 +552,7 @@ static int is_nomap_match(jk_uri_worker_map_t *uw_map,
             (uwr->match_type & MATCH_TYPE_DISABLED))
             continue;
         /* Check only matching workers */
-        if (strcmp(uwr->worker_name, worker))
+        if (*uwr->worker_name != '*' && strcmp(uwr->worker_name, worker))
             continue;
         if (uwr->match_type & MATCH_TYPE_WILDCHAR_PATH) {
             /* Map is already sorted by context_len */
@@ -532,9 +588,11 @@ static int is_nomap_match(jk_uri_worker_map_t *uw_map,
 
 
 const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
-                              const char *uri, jk_logger_t *l)
+                              const char *uri, const char *vhost,
+                              jk_logger_t *l)
 {
     unsigned int i;
+    unsigned int vhost_len;
     int reject_unsafe;
     const char *rv = NULL;
     char  url[JK_MAX_URI_LEN+1];
@@ -553,7 +611,7 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
         return NULL;
     }
     if (uw_map->fname) {
-        uri_worker_map_update(uw_map, l);
+        uri_worker_map_update(uw_map, 0, l);
         if (!uw_map->size) {
             jk_log(l, JK_LOG_INFO,
                    "No worker maps defined for %s.",
@@ -566,10 +624,37 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
      * everything after the first ';' char.
      */
     reject_unsafe = uw_map->reject_unsafe;
+    vhost_len = 0;
+/*
+ * In case we got a vhost, we prepend a slash
+ * and the vhost to the url in order to enable
+ * vhost mapping rules especially for IIS.
+ */
+    if (vhost) {
+        int off = 0;
+/* Add leading '/' if necessary */
+        if (vhost[0] != '/') {
+            url[0] = '/';
+            off = 1;
+        }
+/* Size including leading slash. */
+        vhost_len = strlen(vhost);
+        if (vhost_len + off >= JK_MAX_URI_LEN) {
+            vhost_len = 0;
+            jk_log(l, JK_LOG_WARNING,
+                   "Host prefix %s for URI %s is invalid and will be ignored."
+                   " It must be smaller than %d chars",
+                   vhost, JK_MAX_URI_LEN - off);
+        }
+        else {
+            strncpy(&url[off], vhost, vhost_len + 1);
+        }
+        vhost_len += off;
+    }
     for (i = 0; i < strlen(uri); i++) {
         if (i == JK_MAX_URI_LEN) {
             jk_log(l, JK_LOG_WARNING,
-                   "Uri %s is invalid. Uri must be smaller then %d chars",
+                   "URI %s is invalid. URI must be smaller than %d chars",
                    uri, JK_MAX_URI_LEN);
             JK_TRACE_EXIT(l);
             return NULL;
@@ -577,15 +662,15 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
         if (uri[i] == ';')
             break;
         else {
-            url[i] = uri[i];
-            if (reject_unsafe && (url[i] == '%' || url[i] == '\\')) {
+            url[i + vhost_len] = uri[i];
+            if (reject_unsafe && (uri[i] == '%' || uri[i] == '\\')) {
                 jk_log(l, JK_LOG_INFO, "Potentially unsafe request url '%s' rejected", uri);
                 JK_TRACE_EXIT(l);
                 return NULL;
             }
         }
     }
-    url[i] = '\0';
+    url[i + vhost_len] = '\0';
 
     if (JK_IS_DEBUG_LEVEL(l)) {
         char *url_rewrite = strstr(uri, JK_PATH_SESSION_IDENTIFIER);
@@ -596,64 +681,30 @@ const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
     if (JK_IS_DEBUG_LEVEL(l))
         jk_log(l, JK_LOG_DEBUG, "Attempting to map URI '%s' from %d maps",
                url, uw_map->size);
-
-    for (i = 0; i < uw_map->size; i++) {
-        uri_worker_record_t *uwr = uw_map->maps[i];
-
-        /* Check for match types */
-        if ((uwr->match_type & MATCH_TYPE_DISABLED) ||
-            (uwr->match_type & MATCH_TYPE_NO_MATCH))
-            continue;
-
-        if (JK_IS_DEBUG_LEVEL(l))
-            jk_log(l, JK_LOG_DEBUG, "Attempting to map context URI '%s=%s' source '%s'",
-                   uwr->context, uwr->worker_name, uri_worker_map_get_source(uwr, l));
-
-        if (uwr->match_type & MATCH_TYPE_WILDCHAR_PATH) {
-            const char *wname;
-            /* Map is already sorted by context_len */
-            if (wildchar_match(url, uwr->context,
-#ifdef WIN32
-                               0
-#else
-                               0
-#endif
-                               ) == 0) {
-                    wname = uwr->worker_name;
-                    if (JK_IS_DEBUG_LEVEL(l))
-                        jk_log(l, JK_LOG_DEBUG,
-                               "Found a wildchar match '%s=%s'",
-                               uwr->context, uwr->worker_name);
-                    JK_TRACE_EXIT(l);
-                    rv = wname;
-                    goto cleanup;
-             }
-        }
-        else if (JK_STRNCMP(uwr->context, url, uwr->context_len) == 0) {
-            if (strlen(url) == uwr->context_len) {
-                if (JK_IS_DEBUG_LEVEL(l))
-                    jk_log(l, JK_LOG_DEBUG,
-                           "Found an exact match '%s=%s'",
-                           uwr->context, uwr->worker_name);
-                JK_TRACE_EXIT(l);
-                rv = uwr->worker_name;
-                goto cleanup;
-            }
-        }
+    rv = find_match(uw_map, url, l);
+/* If this doesn't find a match, try without the vhost. */
+    if (! rv && vhost_len) {
+        rv = find_match(uw_map, &url[vhost_len], l);
     }
-    /* No matches found */
-    JK_TRACE_EXIT(l);
 
-cleanup:
+/* In case we found a match, check for the unmounts. */
     if (rv && uw_map->nosize) {
-        if (is_nomap_match(uw_map, url, rv, l)) {
-            if (JK_IS_DEBUG_LEVEL(l))
+/* Again first including vhost. */
+        int rc = is_nomatch(uw_map, url, rv, l);
+/* If no unmount was find, try without vhost. */
+        if (!rc && vhost_len)
+            rc = is_nomatch(uw_map, &url[vhost_len], rv, l);
+        if (rc) {
+            if (JK_IS_DEBUG_LEVEL(l)) {
                 jk_log(l, JK_LOG_DEBUG,
-                       "Denying matching for worker %s by nomatch rule",
+                       "Denying match for worker %s by nomatch rule",
                        rv);
+            }
             rv = NULL;
         }
     }
+
+    JK_TRACE_EXIT(l);
     return rv;
 }
 
@@ -715,12 +766,13 @@ int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
 }
 
 int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
-                          jk_logger_t *l)
+                          int force, jk_logger_t *l)
 {
     int rc = JK_TRUE;
     time_t now = time(NULL);
 
-    if (uw_map->reload > 0 && difftime(now, uw_map->checked) > uw_map->reload) {
+    if ((uw_map->reload > 0 && difftime(now, uw_map->checked) > uw_map->reload) ||
+        force) {
         struct stat statbuf;
         uw_map->checked = now;
         if ((rc = jk_stat(uw_map->fname, &statbuf)) == -1) {
