@@ -18,7 +18,7 @@
 /***************************************************************************
  * Description: URI to worker mapper header file                           *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
- * Version:     $Revision: 549885 $                                           *
+ * Version:     $Revision: 597848 $                                           *
  ***************************************************************************/
 
 #ifndef JK_URI_WORKER_MAP_H
@@ -141,13 +141,14 @@ int uri_worker_map_add(jk_uri_worker_map_t *uw_map,
                        unsigned int source_type, jk_logger_t *l);
 
 const char *map_uri_to_worker(jk_uri_worker_map_t *uw_map,
-                              const char *uri, jk_logger_t *l);
+                              const char *uri, const char *vhost,
+                              jk_logger_t *l);
 
 int uri_worker_map_load(jk_uri_worker_map_t *uw_map,
                         jk_logger_t *l);
 
 int uri_worker_map_update(jk_uri_worker_map_t *uw_map,
-                          jk_logger_t *l);
+                          int force, jk_logger_t *l);
 
 #ifdef __cplusplus
 }
