@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -18,7 +19,7 @@
  * Description: Workers controller                                         *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  * Author:      Henri Gomez <hgomez@apache.org>                            *
- * Version:     $Revision: 416897 $                                          *
+ * Version:     $Revision: 555180 $                                          *
  ***************************************************************************/
 
 #define _PLACE_WORKER_LIST_HERE
@@ -55,12 +56,13 @@ int wc_open(jk_map_t *init_data, jk_worker_env_t *we, jk_logger_t *l)
     JK_INIT_CS(&worker_lock, rc);
     if (rc == JK_FALSE) {
         jk_log(l, JK_LOG_ERROR,
-                "creating thread lock errno=%d",
+                "creating thread lock (errno=%d)",
                 errno);
         JK_TRACE_EXIT(l);
         return JK_FALSE;
     }
 
+    jk_map_dump(init_data, l);
     if (!jk_get_worker_list(init_data, &(we->worker_list),
                             &we->num_of_workers)) {
         JK_TRACE_EXIT(l);
@@ -70,6 +72,8 @@ int wc_open(jk_map_t *init_data, jk_worker_env_t *we, jk_logger_t *l)
     }
 
     worker_maintain_time = jk_get_worker_maintain_time(init_data);
+    if(worker_maintain_time < 0)
+        worker_maintain_time = 0;
 
     if (!build_worker_map(init_data, we->worker_list,
                           we->num_of_workers, we, l)) {
@@ -178,10 +182,11 @@ int wc_create_worker(const char *name, int use_map,
                             "mounting %s to worker %s",
                             map_names[i], name);
                 if (uri_worker_map_add(we->uri_to_worker, map_names[i],
-                                       name, l) == JK_FALSE) {
+                                       name, SOURCE_TYPE_WORKERDEF, l) == JK_FALSE) {
                     w->destroy(&w, l);
                     jk_log(l, JK_LOG_ERROR,
-                           "validate failed for %s", name);
+                           "mounting %s failed for %s",
+                           map_names[i], name);
                     JK_TRACE_EXIT(l);
                     return JK_FALSE;
                 }
@@ -279,6 +284,23 @@ static worker_factory get_factory_for(const char *type)
     return NULL;
 }
 
+const char *wc_get_name_for_type(int type, jk_logger_t *l)
+{
+    worker_factory_record_t *factory = &worker_factories[0];
+    while (factory->name) {
+        if (type == factory->type) {
+            jk_log(l, JK_LOG_DEBUG,
+                   "Found worker type '%s'",
+                   factory->name);
+            return factory->name;
+        }
+
+        factory++;
+    }
+
+    return NULL;
+}
+
 void wc_maintain(jk_logger_t *l)
 {
     static time_t last_maintain = 0;
