@@ -21,7 +21,7 @@
  * Author:      Larry Isaacs <larryi@apache.org>                           *
  * Author:      Ignacio J. Ortega <nacho@apache.org>                       *
  * Author:      Mladen Turk <mturk@apache.org>                             *
- * Version:     $Revision: 521419 $                                        *
+ * Version:     $Revision: 604809 $                                        *
  ***************************************************************************/
 
 // This define is needed to include wincrypt,h, needed to get client certificates
@@ -32,6 +32,7 @@
 #include <wininet.h>
 
 #include "jk_global.h"
+#include "jk_url.h"
 #include "jk_util.h"
 #include "jk_map.h"
 #include "jk_pool.h"
@@ -39,6 +40,7 @@
 #include "jk_worker.h"
 #include "jk_uri_worker_map.h"
 #include "jk_shm.h"
+#include "jk_ajp13.h"
 #include "pcre.h"
 
 #ifndef POSIX_MALLOC_THRESHOLD
@@ -94,11 +96,13 @@ static char HTTP_WORKER_HEADER_NAME[MAX_PATH];
 #define URI_SELECT_PARSED_VERB      ("parsed")
 #define URI_SELECT_UNPARSED_VERB    ("unparsed")
 #define URI_SELECT_ESCAPED_VERB     ("escaped")
+#define URI_SELECT_PROXY_VERB       ("proxy")
 #define URI_REWRITE_TAG             ("rewrite_rule_file")
 #define SHM_SIZE_TAG                ("shm_size")
 #define WORKER_MOUNT_RELOAD_TAG     ("worker_mount_reload")
 #define STRIP_SESSION_TAG           ("strip_session")
 #define AUTH_COMPLETE_TAG           ("auth_complete")
+#define REJECT_UNSAFE_TAG           ("reject_unsafe")
 
 
 #define TRANSLATE_HEADER            ("Translate:")
@@ -108,6 +112,7 @@ static char HTTP_WORKER_HEADER_NAME[MAX_PATH];
 #define BAD_REQUEST     -1
 #define BAD_PATH        -2
 #define MAX_SERVERNAME  128
+#define MAX_PACKET_SIZE 65536
 
 char HTML_ERROR_400[] =         "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n"
                                 "<HTML><HEAD><TITLE>Bad request!</TITLE></HEAD>\n"
@@ -135,12 +140,13 @@ char HTML_ERROR_503[] =         "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Tr
                                 "Please try again later.</P></BODY></HTML>";
 
 
+#define STRNULL_FOR_NULL(x) ((x) ? (x) : "(null)")
 #define JK_TOLOWER(x)   ((char)tolower((BYTE)(x)))
 
 #define GET_SERVER_VARIABLE_VALUE(name, place)          \
   do {                                                  \
     (place) = NULL;                                     \
-    huge_buf_sz = sizeof(huge_buf);                     \
+    huge_buf_sz = MAX_PACKET_SIZE;                      \
     if (get_server_value(private_data->lpEcb,           \
                         (name),                         \
                         huge_buf,                       \
@@ -151,7 +157,7 @@ char HTML_ERROR_503[] =         "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Tr
 
 #define GET_SERVER_VARIABLE_VALUE_INT(name, place, def)     \
   do {                                                      \
-    huge_buf_sz = sizeof(huge_buf);                         \
+    huge_buf_sz = MAX_PACKET_SIZE;                          \
     if (get_server_value(private_data->lpEcb,               \
                         (name),                             \
                         huge_buf,                           \
@@ -191,12 +197,14 @@ static int shm_config_size = JK_SHM_DEF_SIZE;
 static int strip_session = 0;
 static DWORD auth_notification_flags = 0;
 static int   use_auth_notification_flags = 1;
+static int reject_unsafe = 0;
 
 #define URI_SELECT_OPT_PARSED       0
 #define URI_SELECT_OPT_UNPARSED     1
 #define URI_SELECT_OPT_ESCAPED      2
+#define URI_SELECT_OPT_PROXY        3
 
-static int uri_select_option = URI_SELECT_OPT_PARSED;
+static int uri_select_option = URI_SELECT_OPT_PROXY;
 
 static jk_worker_env_t worker_env;
 
@@ -205,7 +213,6 @@ struct isapi_private_data_t
 {
     jk_pool_t p;
 
-    int request_started;
     unsigned int bytes_read_so_far;
     LPEXTENSION_CONTROL_BLOCK lpEcb;
 };
@@ -580,12 +587,12 @@ static int JK_METHOD start_response(jk_ws_service_t *s,
     if (s && s->ws_private) {
         int rv = JK_TRUE;
         isapi_private_data_t *p = s->ws_private;
-        if (!p->request_started) {
+        if (!s->response_started) {
             char *status_str;
             DWORD status_str_len;
             char *headers_str = NULL;
             BOOL keep_alive = FALSE;
-            p->request_started = JK_TRUE;
+            s->response_started = JK_TRUE;
 
             /*
              * Create the status line
@@ -736,7 +743,7 @@ static int JK_METHOD write(jk_ws_service_t *s, const void *b, unsigned int l)
             unsigned int written = 0;
             char *buf = (char *)b;
 
-            if (!p->request_started) {
+            if (!s->response_started) {
                 start_response(s, 200, NULL, NULL, NULL, 0);
             }
 
@@ -1121,6 +1128,7 @@ static int rregex_rewrite(char *uri)
     return 0;
 }
 
+static int init_error = 0;
 
 DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                             DWORD dwNotificationType, LPVOID pvNotification)
@@ -1140,6 +1148,14 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
         if (!is_mapread)
             is_inited = JK_FALSE;
     }
+    if (!is_inited && !is_mapread) {
+        /* In case the initialization failed
+         * return error. This will make entire IIS
+         * unusable like with Apache servers
+         */
+         SetLastError(ERROR_INVALID_FUNCTION);
+         return SF_STATUS_REQ_ERROR;
+    }
     if (auth_notification_flags == dwNotificationType) {
         char uri[INTERNET_MAX_URL_LENGTH];
         char snuri[INTERNET_MAX_URL_LENGTH] = "/";
@@ -1245,19 +1261,10 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
             }
             if (szHost > 0) {
                 StringCbCat(snuri, INTERNET_MAX_URL_LENGTH, Host);
-                StringCbCat(snuri, INTERNET_MAX_URL_LENGTH, uri);
-                if (JK_IS_DEBUG_LEVEL(logger))
-                    jk_log(logger, JK_LOG_DEBUG,
-                           "Virtual Host redirection of %s",
-                           snuri);
-                worker = map_uri_to_worker(uw_map, snuri, logger);
+                worker = map_uri_to_worker(uw_map, uri, snuri, logger);
             }
-            if (!worker) {
-                if (JK_IS_DEBUG_LEVEL(logger))
-                    jk_log(logger, JK_LOG_DEBUG,
-                           "Default redirection of %s",
-                           uri);
-                worker = map_uri_to_worker(uw_map, uri, logger);
+            else {
+                worker = map_uri_to_worker(uw_map, uri, NULL, logger);
             }
             /*
              * Check if somebody is feading us with his own TOMCAT data headers.
@@ -1268,13 +1275,13 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
 
                 if (JK_IS_DEBUG_LEVEL(logger))
                     jk_log(logger, JK_LOG_DEBUG,
-                           "check if [%s] is points to the web-inf directory",
+                           "check if [%s] points to the web-inf directory",
                         uri);
 
                 if (uri_is_web_inf(uri)) {
                     jk_log(logger, JK_LOG_EMERG,
                            "[%s] points to the web-inf or meta-inf directory. "
-                           "Somebody try to hack into the site!!!",
+                           "Somebody tries to hack into the site!!!",
                            uri);
 
                     write_error_response(pfc, "404 Not Found",
@@ -1297,7 +1304,7 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                         *(query - 1) = '\0';
                     if (JK_IS_DEBUG_LEVEL(logger))
                         jk_log(logger, JK_LOG_DEBUG,
-                               "fowarding original URI [%s]",
+                               "forwarding original URI [%s]",
                                uri);
                     forwardURI = uri;
                 }
@@ -1316,6 +1323,21 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                                snuri);
                     forwardURI = snuri;
                 }
+                else if (uri_select_option == URI_SELECT_OPT_PROXY) {
+                    if (!jk_canonenc(uri, snuri, INTERNET_MAX_URL_LENGTH)) {
+                        jk_log(logger, JK_LOG_ERROR,
+                               "[%s] re-encoding request exceeds maximum buffer size.",
+                               uri);
+                        write_error_response(pfc, "400 Bad Request",
+                                             HTML_ERROR_400);
+                        return SF_STATUS_REQ_FINISHED;
+                    }
+                    if (JK_IS_DEBUG_LEVEL(logger))
+                        jk_log(logger, JK_LOG_DEBUG,
+                               "fowarding escaped URI [%s]",
+                               snuri);
+                    forwardURI = snuri;
+                }
                 else {
                     forwardURI = uri;
                 }
@@ -1330,12 +1352,12 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                     StringCbCopy(duri, INTERNET_MAX_URL_LENGTH, forwardURI);
                     if (simple_rewrite(forwardURI)) {
                         jk_log(logger, JK_LOG_DEBUG,
-                               "rewriten URI [%s]->[%s]",
+                               "rewritten URI [%s]->[%s]",
                                duri, forwardURI);
                     }
                     else if (rregex_rewrite(forwardURI)) {
                         jk_log(logger, JK_LOG_DEBUG,
-                               "rewriten URI [%s]->[%s]",
+                               "rewritten URI [%s]->[%s]",
                                duri, forwardURI);
                     }
                 }
@@ -1351,6 +1373,7 @@ DWORD WINAPI HttpFilterProc(PHTTP_FILTER_CONTEXT pfc,
                     !SetHeader(pfc, "url", extension_uri)) {
                     jk_log(logger, JK_LOG_ERROR,
                            "error while adding request headers");
+                    SetLastError(ERROR_INVALID_PARAMETER);
                     return SF_STATUS_REQ_ERROR;
                 }
 
@@ -1467,7 +1490,6 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
         jk_init_ws_service(&s);
         jk_open_pool(&private_data.p, buf, sizeof(buf));
 
-        private_data.request_started = JK_FALSE;
         private_data.bytes_read_so_far = 0;
         private_data.lpEcb = lpEcb;
 
@@ -1488,7 +1510,8 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
                 s.retries = worker->retries;
                 if (worker->get_endpoint(worker, &e, logger)) {
                     int is_error = JK_HTTP_SERVER_ERROR;
-                    if (e->service(e, &s, logger, &is_error)) {
+                    int result;
+                    if ((result = e->service(e, &s, logger, &is_error)) > 0) {
                         rc = HSE_STATUS_SUCCESS;
                         lpEcb->dwHttpStatusCode = HTTP_STATUS_OK;
                         if (JK_IS_DEBUG_LEVEL(logger))
@@ -1496,8 +1519,14 @@ DWORD WINAPI HttpExtensionProc(LPEXTENSION_CONTROL_BLOCK lpEcb)
                                    "service() returned OK");
                     }
                     else {
-                        jk_log(logger, JK_LOG_ERROR,
-                               "service() failed with http error %d", is_error);
+                        if ((result == JK_CLIENT_ERROR) && (is_error == JK_HTTP_OK)) {
+                            jk_log(logger, JK_LOG_INFO,
+                                   "service() failed because client aborted connection");
+                        }
+                        else {
+                            jk_log(logger, JK_LOG_ERROR,
+                                   "service() failed with http error %d", is_error);
+                        }
                         lpEcb->dwHttpStatusCode = is_error;
                         write_error_message(lpEcb, is_error);
                     }
@@ -1566,6 +1595,7 @@ BOOL WINAPI TerminateFilter(DWORD dwFlags)
             jk_map_free(&rregexp_map);
         }
         wc_close(logger);
+        jk_shm_close();
         if (logger) {
             jk_close_file_logger(&logger);
         }
@@ -1670,7 +1700,8 @@ static int init_jk(char *serverName)
     }
 
     if (rewrite_rule_file[0] && jk_map_alloc(&rewrite_map)) {
-        if (jk_map_load_properties(rewrite_map, rewrite_rule_file, NULL, logger)) {
+        if (jk_map_read_properties(rewrite_map, rewrite_rule_file,
+                                   NULL, JK_MAP_HANDLE_RAW, logger)) {
             int i;
             if (JK_IS_DEBUG_LEVEL(logger)) {
                 jk_log(logger, JK_LOG_DEBUG, "Loaded rewrite rule file %s.",
@@ -1711,6 +1742,10 @@ static int init_jk(char *serverName)
 
     if (uri_worker_map_alloc(&uw_map, NULL, logger)) {
         rc = JK_FALSE;
+        if (reject_unsafe)
+            uw_map->reject_unsafe = 1;
+        else
+            uw_map->reject_unsafe = 0;
         uw_map->fname = worker_mount_file;
         uw_map->reload = worker_mount_reload;
         if (worker_mount_file[0])
@@ -1719,7 +1754,8 @@ static int init_jk(char *serverName)
     if (rc) {
         rc = JK_FALSE;
         if (jk_map_alloc(&workers_map)) {
-            if (jk_map_read_properties(workers_map, worker_file, NULL, 1, logger)) {
+            if (jk_map_read_properties(workers_map, worker_file, NULL,
+                                       JK_MAP_HANDLE_DUPLICATES, logger)) {
                 /* we add the URI->WORKER MAP since workers using AJP14 will feed it */
 
                 if (jk_map_resolve_references(workers_map, "worker.", 1, 1, logger) == JK_FALSE) {
@@ -1775,6 +1811,10 @@ int parse_uri_select(const char *uri_select)
         return URI_SELECT_OPT_ESCAPED;
     }
 
+    if (0 == strcasecmp(uri_select, URI_SELECT_PROXY_VERB)) {
+        return URI_SELECT_OPT_PROXY;
+    }
+
     return -1;
 }
 
@@ -1787,7 +1827,8 @@ static int read_registry_init_data(void)
     jk_map_t *map = NULL;
 
     if (jk_map_alloc(&map)) {
-        if (jk_map_read_properties(map, ini_file_name, NULL, 1, logger)) {
+        if (jk_map_read_properties(map, ini_file_name, NULL,
+                                   JK_MAP_HANDLE_DUPLICATES, logger)) {
             using_ini_file = JK_TRUE;
             src = map;
         }
@@ -1826,6 +1867,7 @@ static int read_registry_init_data(void)
     worker_mount_reload = get_config_int(src, WORKER_MOUNT_RELOAD_TAG, JK_URIMAP_DEF_RELOAD);
     strip_session = get_config_bool(src, STRIP_SESSION_TAG, JK_FALSE);
     use_auth_notification_flags = get_config_int(src, AUTH_COMPLETE_TAG, 1);
+    reject_unsafe = get_config_bool(src, REJECT_UNSAFE_TAG, JK_FALSE);
     if (using_ini_file) {
         jk_map_free(&map);
     }
@@ -1922,7 +1964,7 @@ static int get_registry_config_number(HKEY hkey,
 static int init_ws_service(isapi_private_data_t * private_data,
                            jk_ws_service_t *s, char **worker_name)
 {
-    char huge_buf[16 * 1024];   /* should be enough for all */
+    char *huge_buf = NULL;   /* should be enough for all */
 
     DWORD huge_buf_sz;
 
@@ -1942,6 +1984,11 @@ static int init_ws_service(isapi_private_data_t * private_data,
     /* Clear RECO status */
     s->reco_status = RECO_NONE;
 
+    if (!(huge_buf = jk_pool_alloc(&private_data->p, MAX_PACKET_SIZE))) {
+
+        return JK_FALSE;    
+    }
+    huge_buf_sz = MAX_PACKET_SIZE;
     GET_SERVER_VARIABLE_VALUE(HTTP_WORKER_HEADER_NAME, (*worker_name));
     GET_SERVER_VARIABLE_VALUE(HTTP_URI_HEADER_NAME, s->req_uri);
     GET_SERVER_VARIABLE_VALUE(HTTP_QUERY_HEADER_NAME, s->query_string);
@@ -1966,7 +2013,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
     GET_SERVER_VARIABLE_VALUE_INT("SERVER_PORT_SECURE", s->is_ssl, 0);
 
     s->method = private_data->lpEcb->lpszMethod;
-    s->content_length = private_data->lpEcb->cbTotalBytes;
+    s->content_length = (jk_uint64_t)private_data->lpEcb->cbTotalBytes;
 
     s->ssl_cert = NULL;
     s->ssl_cert_len = 0;
@@ -2032,7 +2079,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
             s->num_attributes = num_of_vars;
             if (ssl_env_values[4] && ssl_env_values[4][0] == '1') {
                 CERT_CONTEXT_EX cc;
-                cc.cbAllocated = sizeof(huge_buf);
+                cc.cbAllocated = MAX_PACKET_SIZE;
                 cc.CertContext.pbCertEncoded = (BYTE *) huge_buf;
                 cc.CertContext.cbCertEncoded = 0;
 
@@ -2061,7 +2108,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
         }
     }
 
-    huge_buf_sz = sizeof(huge_buf);
+    huge_buf_sz = MAX_PACKET_SIZE;
     if (get_server_value(private_data->lpEcb,
                          "ALL_HTTP", huge_buf, huge_buf_sz)) {
         unsigned int cnt = 0;
@@ -2074,7 +2121,7 @@ static int init_ws_service(isapi_private_data_t * private_data,
         }
 
         if (cnt) {
-            char *headers_buf = jk_pool_strdup(&private_data->p, huge_buf);
+            char *headers_buf = huge_buf;
             unsigned int i;
             size_t len_of_http_prefix = strlen("HTTP_");
             BOOL need_content_length_header = (s->content_length == 0);
@@ -2182,6 +2229,23 @@ static int init_ws_service(isapi_private_data_t * private_data,
         return JK_FALSE;
     }
 
+    /* Dump all connection param so we can trace what's going to
+     * the remote tomcat
+     */
+    if (JK_IS_DEBUG_LEVEL(logger)) {
+        jk_log(logger, JK_LOG_DEBUG,
+               "Service protocol=%s method=%s host=%s addr=%s name=%s port=%d auth=%s user=%s uri=%s",
+               STRNULL_FOR_NULL(s->protocol),
+               STRNULL_FOR_NULL(s->method),
+               STRNULL_FOR_NULL(s->remote_host),
+               STRNULL_FOR_NULL(s->remote_addr),
+               STRNULL_FOR_NULL(s->server_name),
+               s->server_port,
+               STRNULL_FOR_NULL(s->auth_type),
+               STRNULL_FOR_NULL(s->remote_user),
+               STRNULL_FOR_NULL(s->req_uri));
+    }
+
     return JK_TRUE;
 }
 
