@@ -1,9 +1,10 @@
 /*
- *  Copyright 1999-2004 The Apache Software Foundation
- *
- *  Licensed under the Apache License, Version 2.0 (the "License");
- *  you may not use this file except in compliance with the License.
- *  You may obtain a copy of the License at
+ *  Licensed to the Apache Software Foundation (ASF) under one or more
+ *  contributor license agreements.  See the NOTICE file distributed with
+ *  this work for additional information regarding copyright ownership.
+ *  The ASF licenses this file to You under the Apache License, Version 2.0
+ *  (the "License"); you may not use this file except in compliance with
+ *  the License.  You may obtain a copy of the License at
  *
  *      http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -18,7 +19,7 @@
  * Description: NT System service for Tomcat                       *
  * Author:      Gal Shachor <shachor@il.ibm.com>                           *
  *              Dave Oxley <Dave@JungleMoss.com>                           *
- * Version:     $Revision: 417906 $                                           *
+ * Version:     $Revision: 603324 $                                           *
  ***************************************************************************/
 
 #include "jk_global.h"
@@ -381,7 +382,7 @@ void install_service(char *name,
         if (dst >= szTrueName + sizeof(szTrueName) - 1) {
             break;
         }
-        if (!isspace(*src) && *src != '/' && *src != '\\') {
+        if (!isspace((int)(*src)) && *src != '/' && *src != '\\') {
             *(dst++) = *src;
         }
     }
@@ -870,8 +871,8 @@ static void stop_tomcat(char *name,
         return;
     } 
     
-    if(jk_resolve("localhost", port, &in)) {
-        int sd = jk_open_socket(&in, JK_TRUE, 0, -1, NULL);
+    if(jk_resolve("localhost", port, &in, NULL)) {
+        int sd = jk_open_socket(&in, JK_TRUE, 0, 0, NULL);
         if(sd >0) {
             int rc = JK_FALSE;
 
@@ -909,7 +910,8 @@ static void stop_tomcat(char *name,
     
                     if(0 > jk_tcp_socket_sendfull(sd, 
                                                   msg->buf,
-                                                  msg->len)) {
+                                                  msg->len,
+                                                  NULL)) {
                         rc = JK_FALSE;
                     }
                 }                                                    
@@ -920,7 +922,7 @@ static void stop_tomcat(char *name,
                     rc = JK_TRUE;
                 }
             }
-            jk_close_socket(sd);
+            jk_close_socket(sd, NULL);
             if(JK_TRUE == rc) {
                 if(WAIT_OBJECT_0 == WaitForSingleObject(hTomcat, 30*1000)) {
                     return;
@@ -955,7 +957,8 @@ static int exec_cmd(const char *name, HANDLE *hTomcat, char *cmdLine)
             jk_map_t *init_map;
             
             if(jk_map_alloc(&init_map)) {
-                if(jk_map_read_properties(init_map, prp_file, NULL)) {
+                if(jk_map_read_properties(init_map, prp_file, NULL,
+                                          JK_MAP_HANDLE_DUPLICATES, NULL)) {
                     jk_tomcat_startup_data_t data;
                     jk_pool_t p;
                     jk_pool_atom_t buf[HUGE_POOL_SIZE];
@@ -1000,8 +1003,8 @@ static int exec_cmd(const char *name, HANDLE *hTomcat, char *cmdLine)
 
                         memset(&processInformation, 0, sizeof(processInformation));
                         
-						if( cmdLine==NULL ) 
-							cmdLine=data.cmd_line;
+                        if( cmdLine==NULL ) 
+                            cmdLine=data.cmd_line;
 
                         printf(cmdLine);
                         if(CreateProcess(data.java_bin,
@@ -1023,7 +1026,7 @@ static int exec_cmd(const char *name, HANDLE *hTomcat, char *cmdLine)
                             shutdown_port = data.shutdown_port;
                             shutdown_secret = data.shutdown_secret;
                             shutdown_protocol = strdup(data.shutdown_protocol);
-							shutdown_cmd = strdup(data.stop_cmd);
+                            shutdown_cmd = strdup(data.stop_cmd);
 
                             return JK_TRUE;
                         } else {
