@@ -1,5 +1,21 @@
 #!/bin/ksh
 
+# Licensed to the Apache Software Foundation (ASF) under one or more
+# contributor license agreements.  See the NOTICE file distributed with
+# this work for additional information regarding copyright ownership.
+# The ASF licenses this file to You under the Apache License, Version 2.0
+# (the "License"); you may not use this file except in compliance with
+# the License.  You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+
 # Make sure to set your path so that we can find
 # the following binaries:
 # cd, mkdir, cp, rm, find
@@ -8,28 +24,7 @@
 # libtoolize, aclocal, autoheader, automake, autoconf
 # tar, zip, gzip
 # gpg
-# And any one of: w3m, elinks, links
-
-usage() {
-    echo "Usage:: $0 -t VERSION [-b BRANCH | -T]"
-    echo "        -t: version to package"
-    echo "        -b: package from branch BRANCH"
-    echo "        -T: package from trunk"
-}
-
-while getopts :t:b:T c
-do
-    case $c in
-    t)         tag=$OPTARG;;
-    b)         branch=$OPTARG;;
-    T)         trunk=trunk;;
-    \:)        usage
-               exit 2;;
-    \?)        usage
-               exit 2;;
-    esac
-done
-shift `expr $OPTIND - 1`
+# And any one of: w3m, elinks, links (links2)
 
 SVNROOT="http://svn.apache.org/repos/asf"
 SVNPROJ="tomcat/connectors"
@@ -40,25 +35,102 @@ JK_GROUP="bin"
 
 COPY_TOP="KEYS LICENSE NOTICE"
 COPY_JK="BUILD.txt native jkstatus support tools xdocs"
+COPY_BUILD="docs"
 COPY_CONF="uriworkermap.properties workers.properties workers.properties.minimal"
 
 #################### NO CHANGE BELOW THIS LINE ##############
 
+#################### FUNCTIONS ##############
+
+usage() {
+    echo "Usage:: $0 -t VERSION [-r revision] [-b BRANCH | -T | -d DIR]"
+    echo "        -t: version to package"
+    echo "        -r: revision to package"
+    echo "        -b: package from branch BRANCH"
+    echo "        -T: package from trunk"
+    echo "        -d: package from local directory"
+}
+
+copy_files() {
+    src=$1
+    target=$2
+    list="$3"
+
+    mkdir -p $target
+    for item in $list
+    do
+        echo "Copying $item from $src ..."
+        cp -pr $src/$item $target/
+    done
+}
+
+sign_and_verify() {
+    item=$1
+    echo "Signing $item..."
+    gpg -ba $item
+    echo "Verifying signature for $item..."
+    gpg --verify $item.asc
+}
+
+#################### MAIN ##############
+
+conflict=0
+while getopts :t:r:b:d:T c
+do
+    case $c in
+    t)         tag=$OPTARG;;
+    r)         revision=$OPTARG;;
+    b)         branch=$OPTARG
+               conflict=$(($conflict+1));;
+    T)         trunk=trunk
+               conflict=$(($conflict+1));;
+    d)         local_dir=$OPTARG
+               conflict=$(($conflict+1));;
+    \:)        usage
+               exit 2;;
+    \?)        usage
+               exit 2;;
+    esac
+done
+shift `expr $OPTIND - 1`
+
+if [ $conflict -gt 1 ]
+then
+    usage
+    echo "Only one of the options '-b', '-T'  and '-d' is allowed."
+    exit 2
+fi
+
+if [ -n "$local_dir" ]
+then
+    echo "Caution: Packaging from directory!"
+    echo "Make sure the directory is committed."
+    answer="x"
+    while [ "$answer" != "y" -a "$answer" != "n" ]
+    do
+        echo "Do you want to procede? [y/n]"
+        read answer
+    done
+    if [ "$answer" != "y" ]
+    then
+        echo "Aborting."
+        exit 4
+    fi
+fi
+
 if [ -z "$tag" ]
 then
     usage
     exit 2
 fi
-if [ -n "$trunk" -a -n "$branch" ]
+if [ -n "$revision" ]
 then
-    usage
-    echo "Only one of the options '-b' and '-T' allowed."
-    exit 2
+    revision="-r $revision"
 fi
 if [ -n "$trunk" ]
 then
     JK_SVN_URL="${SVNROOT}/${SVNPROJ}/trunk"
-    JK_REV=`svn info ${JK_SVN_URL} | awk '$1 == "Revision:" {print $2}'`
+    JK_REV=`svn info $revision ${JK_SVN_URL} | awk '$1 == "Revision:" {print $2}'`
     if [ -z "$JK_REV" ]
     then
        echo "No Revision found at '$JK_SVN_URL'"
@@ -69,13 +141,23 @@ elif [ -n "$branch" ]
 then
     JK_BRANCH=`echo $branch | sed -e 's#/#__#g'`
     JK_SVN_URL="${SVNROOT}/${SVNPROJ}/branches/$branch"
-    JK_REV=`svn info ${JK_SVN_URL} | awk '$1 == "Revision:" {print $2}'`
+    JK_REV=`svn info $revision ${JK_SVN_URL} | awk '$1 == "Revision:" {print $2}'`
     if [ -z "$JK_REV" ]
     then
        echo "No Revision found at '$JK_SVN_URL'"
        exit 3
     fi
     JK_DIST=${JK_CVST}-${tag}-dev-${JK_BRANCH}-${JK_REV}-src
+elif [ -n "$local_dir" ]
+then
+    JK_SVN_URL="$local_dir"
+    JK_REV=`svn info $revision ${JK_SVN_URL} | awk '$1 == "Revision:" {print $2}'`
+    if [ -z "$JK_REV" ]
+    then
+       echo "No Revision found at '$JK_SVN_URL'"
+       exit 3
+    fi
+    JK_DIST=${JK_CVST}-${tag}-dev-local-`date +%Y%m%d%H%M%S`-${JK_REV}-src
 else
     JK_VER=$tag
     JK_TAG=`echo $tag | sed -e 's#^#JK_#' -e 's#\.#_#g'`
@@ -90,10 +172,10 @@ rm -rf ${JK_DIST}
 rm -rf ${JK_DIST}.tmp
 
 mkdir -p ${JK_DIST}.tmp
-svn export "${JK_SVN_URL}/jk" ${JK_DIST}.tmp/jk
+svn export $revision "${JK_SVN_URL}/jk" ${JK_DIST}.tmp/jk
 for item in ${COPY_TOP}
 do
-    svn export "${JK_SVN_URL}/${item}" ${JK_DIST}.tmp/${item}
+    svn export $revision "${JK_SVN_URL}/${item}" ${JK_DIST}.tmp/${item}
 done
 
 # Build documentation.
@@ -101,42 +183,11 @@ cd ${JK_DIST}.tmp/jk/xdocs
 ant
 cd ../../..
 
-# Copying things into source distribution
-srcdir=${JK_DIST}.tmp
-targetdir=${JK_DIST}
-mkdir -p ${targetdir}
-for item in ${COPY_TOP}
-do
-    echo "Copying $item from ${srcdir} ..."
-    cp -pr ${srcdir}/$item ${targetdir}/
-done
-
-srcdir=${JK_DIST}.tmp/jk
-targetdir=${JK_DIST}
-mkdir -p ${targetdir}
-for item in ${COPY_JK}
-do
-    echo "Copying $item from ${srcdir} ..."
-    cp -pr ${srcdir}/$item ${targetdir}/
-done
-
-srcdir=${JK_DIST}.tmp/jk/build
-targetdir=${JK_DIST}
-mkdir -p ${targetdir}
-for item in docs
-do
-    echo "Copying $item from ${srcdir} ..."
-    cp -pr ${srcdir}/$item ${targetdir}/
-done
-
-srcdir=${JK_DIST}.tmp/jk/conf
-targetdir=${JK_DIST}/conf
-mkdir -p ${targetdir}
-for item in ${COPY_CONF}
-do
-    echo "Copying $item from ${srcdir} ..."
-    cp -pr ${srcdir}/$item ${targetdir}/
-done
+# Copying things into the source distribution
+copy_files ${JK_DIST}.tmp $JK_DIST "$COPY_TOP"
+copy_files ${JK_DIST}.tmp/jk $JK_DIST "$COPY_JK"
+copy_files ${JK_DIST}.tmp/jk/build $JK_DIST "$COPY_BUILD"
+copy_files ${JK_DIST}.tmp/jk/conf $JK_DIST/conf "$COPY_CONF"
 
 # Remove extra directories and files
 targetdir=${JK_DIST}
@@ -151,8 +202,8 @@ cd ${JK_DIST}/native
 
 # Check for links, elinks or w3m
 W3MOPTS="-dump -cols 80 -t 4 -S -O iso-8859-1 -T text/html"
-LNKOPTS="-dump -dump-width 80 -dump-charset iso-8859-1 -no-numbering -no-references -no-home"
-ELNKOPTS=$LNKOPTS
+ELNKOPTS="-dump -dump-width 80 -dump-charset iso-8859-1 -no-numbering -no-references -no-home"
+LNKOPTS="-dump -width 80 -codepage iso-8859-1 -no-g -html-numbered-links 0"
 failed=true
 for tool in `echo "w3m elinks links"`
 do
@@ -216,10 +267,13 @@ fi
 ./buildconf.sh
 cd ../../
 
-# Pack and sign
+# Pack
 tar cfz ${JK_DIST}.tar.gz --owner="${JK_OWNER}" --group="${JK_GROUP}" ${JK_DIST}
 perl ${JK_DIST}/tools/lineends.pl --cr ${JK_DIST}
 zip -9 -r ${JK_DIST}.zip ${JK_DIST}
-# Create detatched signature
-gpg -ba ${JK_DIST}.tar.gz
-gpg -ba ${JK_DIST}.zip
+
+# Create detached signature and verify it
+archive=${JK_DIST}.tar.gz
+sign_and_verify $archive
+archive=${JK_DIST}.zip
+sign_and_verify $archive
